Package: IntegMultiReg
Title: Integrative Bayesian Multiple Regression for Multi-Platform
        Biomarkers
Version: 0.1.1
Authors@R: c(
    person("Sinian", "Zhang", email = "zhan9381@umn.edu",
           role = "aut"),
    person("Jianfeng", "Wang", email = "wan01188@umn.edu",
           role = "aut"),
    person("Thierry", "Chekouo", email = "tchekouo@umn.edu",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3594-9182")))
Description: A Bayesian framework that integrates several regression models to
    identify a parsimonious set of biomarkers shared across disparate data
    platforms (for example genomic, transcriptomic and proteomic assays).
    Subjects are partitioned into subgroups defined by their pattern of
    platform availability, so that no subject with partially missing platform
    data is excluded, and information is borrowed across subgroups through a
    Markov random field prior on the variable-selection indicators together
    with non-local (product moment) priors on the regression effects. The
    methodology was introduced for time-to-event outcomes by Chekouo,
    Stingo, Doecke and Do (2017) <doi:10.1111/biom.12587>; this package
    additionally supports continuous (Gaussian) and binary (probit) outcomes.
    Posterior inference is carried out by a Markov chain Monte Carlo sampler
    implemented in C for computational efficiency.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.0.0)
Imports: graphics, grDevices, stats, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
SystemRequirements: GSL (>= 2.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: yes
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.2
Packaged: 2026-08-21 21:52:42 UTC; tchekouo
Author: Sinian Zhang [aut],
  Jianfeng Wang [aut],
  Thierry Chekouo [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3594-9182>)
Maintainer: Thierry Chekouo <tchekouo@umn.edu>
Repository: CRAN
Date/Publication: 2026-08-22 15:20:14 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-08-22 17:32:36 UTC; unix
Archs: IntegMultiReg.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(coef,imr)
S3method(plot,imr)
S3method(predict,imr)
S3method(print,imr)
S3method(print,summary.imr)
S3method(summary,imr)
export(coef_imr)
export(cv_imr)
export(imr)
export(plot_imr)
export(plot_subgroup_sizes)
export(plot_top_features)
export(predict_imr)
export(summary_imr)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,image)
importFrom(graphics,par)
importFrom(stats,pnorm)
importFrom(stats,sd)
importFrom(utils,capture.output)
useDynLib(IntegMultiReg, .registration = TRUE)

# IntegMultiReg

**Integrative Bayesian Multiple Regression for Multi-Platform Biomarkers.**

`IntegMultiReg` implements the integrative multi-regression (IMR) model of
Chekouo, Stingo, Doecke and Do (2017, *Biometrics*) and extends it from
time-to-event outcomes to continuous (Gaussian) and binary (probit) outcomes.

Given several molecular platforms measured on overlapping but partially missing
sets of subjects, IMR partitions subjects into the availability subgroups of a
Venn diagram, fits one regression per subgroup, and shares information across
availability subgroups through

* **non-local (product moment) priors** on the regression coefficients, and
* a **Markov random field (MRF) prior** on the variable-selection indicators,

so that no subject with partially observed platforms is discarded and the same
biomarkers tend to be selected across availability subgroups.

## Installation

The package contains C code that links against the
[GNU Scientific Library (GSL)](https://www.gnu.org/software/gsl/), which must be
installed first:

* macOS: `brew install gsl`
* Debian/Ubuntu: `sudo apt-get install libgsl-dev`
* Windows: GSL is provided by Rtools.

Once available on CRAN, install the package with:

```r
install.packages("IntegMultiReg")
```

Alternatively, install a local source tarball:

```r
install.packages("IntegMultiReg_0.1.0.tar.gz", repos = NULL, type = "source")
```

The CRAN checking tools `checkbashisms` and `qpdf` are not runtime
dependencies. Package users do not need them. Maintainers running
`R CMD check --as-cran` locally can install them with
`brew install checkbashisms qpdf` on macOS or
`sudo apt-get install devscripts qpdf` on Debian/Ubuntu.

## Quick start

```r
library(IntegMultiReg)
data("simIMR")

fit <- imr(
  platform_data_list = simIMR$platforms,
  outcome            = simIMR$outcome.binary,
  cov                = simIMR$covariates,
  type_outcome       = "binary",
  nu                 = c(-4, -3, -4),
  sample_mcmc        = c(2000, 1000),
  ssize              = 30,
  seed               = 1
)

fit                       # short summary
summary(fit)              # selected biomarkers per platform
coef(fit)                 # per-platform mPIP matrices
plot(fit, type = "selection")
plot_top_features(fit)    # ranked biomarker bar chart
predict(fit, newdata = simIMR$platforms[1:2], covariates = simIMR$covariates)
cv_imr(fit)               # fold-split predictive assessment using fitted samples
```

## Real-data example

`kircIMR` is a reduced public UCSC Xena TCGA-KIRC survival example aligned with
the Biometrics kidney cancer case study: mRNA expression, miRNA expression, DNA
methylation, clinical covariates and right-censored survival.  It is derived
from public UCSC Xena TCGA-KIRC sampleMap files, not from controlled-access
TCGA/GDC files, and contains only a reduced Cox-screened feature panel.

The package replaces TCGA barcodes with package-internal IDs such as `KIRC001`
and does not distribute a barcode mapping.  Users should not attempt
participant re-identification or linkage to external resources.

```r
data("kircIMR")
sapply(kircIMR$platforms, dim)
kircIMR$model_subgroup_sizes

kirc_fit <- imr(
  kircIMR$platforms,
  kircIMR$outcome.survival,
  cov = kircIMR$covariates,
  type_outcome = "right.censored",
  nu = c(-4, -3, -4),
  sample_mcmc = c(4000, 1000),
  ssize = 30,
  seed = 1
)
```

See the package vignette `vignette("IntegMultiReg")` for a complete walk-through.

## Reference

Chekouo T, Stingo FC, Doecke JD, Do K-A (2017). "A Bayesian Integrative
Approach for Multi-Platform Genomic Data: A Kidney Cancer Case Study."
*Biometrics*, **73**(2), 615–624. <https://doi.org/10.1111/biom.12587>

When using `kircIMR`, please also acknowledge TCGA, the National Cancer
Institute Genomic Data Commons, and UCSC Xena as the public data sources.

#' IntegMultiReg: Integrative Bayesian Multiple Regression for Multi-Platform Biomarkers
#'
#' The \pkg{IntegMultiReg} package implements the integrative multi-regression
#' (IMR) approach of Chekouo et al. (2017) and extends it from time-to-event
#' outcomes to continuous (Gaussian) and binary (probit) outcomes.  Subjects
#' are partitioned into availability subgroups according to which platforms
#' they have measured, one regression model is built per subgroup, and
#' information is shared across availability subgroups through a Markov random
#' field prior on the
#' variable-selection indicators combined with non-local priors on the
#' regression coefficients.
#'
#' The main entry points are:
#' \describe{
#'   \item{\code{\link{imr}}}{Fit the model by MCMC and return an object of
#'     class \code{"imr"}.}
#'   \item{\code{\link{predict.imr}}}{Predict outcomes for new subjects.}
#'   \item{\code{\link{cv_imr}}}{Assess predictive performance by
#'     cross-validation.}
#' }
#' Standard methods \code{\link{print.imr}}, \code{\link{summary.imr}},
#' \code{\link{coef.imr}}, \code{\link{plot.imr}} and \code{\link{predict.imr}}
#' are provided for the fitted object, together with the stand-alone displays
#' \code{\link{plot_top_features}} and \code{\link{plot_subgroup_sizes}}.
#'
#' Jianfeng Wang initiated the R interface that calls the C implementation.
#'
#' @references
#' Chekouo T, Stingo FC, Doecke JD, Do K-A (2017). "A Bayesian Integrative
#' Approach for Multi-Platform Genomic Data: A Kidney Cancer Case Study."
#' \emph{Biometrics}, \strong{73}(2), 615--624. \doi{10.1111/biom.12587}
#'
#' @keywords internal
#' @useDynLib IntegMultiReg, .registration = TRUE
#' @importFrom stats sd pnorm
#' @importFrom graphics abline axis box image par
#' @importFrom grDevices colorRampPalette
#' @importFrom utils capture.output
"_PACKAGE"

## Shared fixtures for the test suite.  Small MCMC runs keep R CMD check fast
## while still exercising the full fit -> predict -> cv pipeline.

data("simIMR", package = "IntegMultiReg")

fit_demo <- function(type = c("binary", "continuous", "right.censored"),
                     total = 300, burn = 150, seed = 42) {
  type <- match.arg(type)
  outcome <- switch(type,
    binary = simIMR$outcome.binary,
    continuous = simIMR$outcome.continuous,
    right.censored = simIMR$outcome.survival
  )
  imr(
    platform_data_list = simIMR$platforms,
    outcome = outcome,
    cov = simIMR$covariates,
    type_outcome = type,
    nu = c(-4, -3, -4),
    sample_mcmc = c(total, burn),
    ssize = 30,
    seed = seed
  )
}

## A binary fit reused across several test files.
fit_bin <- fit_demo("binary", seed = 42)

test_that("the fitted object has the expected class and structure", {
  expect_s3_class(fit_bin, "imr")
  for (nm in c("gam_mean", "theta_mean", "estimate_latent_y", "log_posterior",
               "gam_sample", "list_hyperpara", "data1", "data2",
               "model_bitstrings", "sample_size", "platform_names",
               "feature_names", "nu", "sample_mcmc")) {
    expect_true(nm %in% names(fit_bin), info = nm)
  }
  expect_equal(fit_bin$n_platform, 3L)
  expect_equal(length(fit_bin$gam_mean), 3L)
})

test_that("subgroup structure matches the simulated availability patterns", {
  expect_setequal(fit_bin$model_bitstrings, c("011", "100", "101", "111"))
  expect_equal(as.integer(fit_bin$sample_size[c("011", "100", "101", "111")]),
               c(120L, 60L, 60L, 60L))
})

test_that("inclusion probabilities are valid and dimensions are right", {
  mp <- coef(fit_bin)
  expect_equal(names(mp), c("genomic", "proteomic", "metabolomic"))
  expect_equal(ncol(mp$genomic), 20L)
  expect_equal(ncol(mp$proteomic), 10L)
  expect_equal(ncol(mp$metabolomic), 8L)
  for (m in mp) {
    expect_true(all(m >= 0 & m <= 1))
    expect_false(anyNA(m))
  }
})

test_that("the log-posterior trace is finite and the right length", {
  expect_length(fit_bin$log_posterior, sum(fit_bin$sample_mcmc))
  expect_true(all(is.finite(fit_bin$log_posterior)))
})

test_that("results are reproducible for a fixed seed", {
  a <- fit_demo("binary", total = 200, burn = 100, seed = 7)
  b <- fit_demo("binary", total = 200, burn = 100, seed = 7)
  expect_equal(a$gam_mean, b$gam_mean)
  expect_equal(a$log_posterior, b$log_posterior)
})

test_that("seed = NULL follows the ambient RNG (set.seed reproducibility)", {
  run <- function() {
    imr(simIMR$platforms, simIMR$outcome.binary, cov = simIMR$covariates,
        type_outcome = "binary", nu = c(-4, -3, -4),
        sample_mcmc = c(120, 60), ssize = 30, seed = NULL)
  }
  set.seed(123); a <- run()
  set.seed(123); b <- run()
  expect_equal(a$log_posterior, b$log_posterior)
})

test_that("the model recovers planted signal above null features", {
  # deterministic given the seed; use a moderate run
  fit <- fit_demo("continuous", total = 800, burn = 300, seed = 5)
  mp <- coef(fit)
  # genomic truth = G01,G02,G03; compare best true vs best null feature
  true_max <- max(apply(mp$genomic[, 1:3, drop = FALSE], 2, max))
  null_max <- max(apply(mp$genomic[, 8:20, drop = FALSE], 2, max))
  expect_gt(true_max, null_max)
  # at least one truly associated feature is confidently selected somewhere
  expect_gt(max(apply(mp$proteomic[, 1:2, drop = FALSE], 2, max)), 0.5)
})

test_that("BMS fits keep theta fixed at zero and return empty theta samples", {
  fit <- imr(
    simIMR$platforms, simIMR$outcome.binary, cov = simIMR$covariates,
    type_outcome = "binary", method = "BMS", nu = c(-4, -3, -4),
    sample_mcmc = c(80, 40), ssize = 30, seed = 16
  )

  expect_true(all(vapply(fit$theta_mean, function(x) all(x == 0), logical(1))))
  expect_true(all(vapply(fit$theta_sample, is.null, logical(1))))
})

test_that("single-subgroup fits handle zero-column theta samples", {
  set.seed(17)
  n <- 24
  platforms <- list(
    first = data.frame(id = seq_len(n), x1 = rnorm(n), x2 = rnorm(n)),
    second = data.frame(id = seq_len(n), z1 = rnorm(n))
  )
  outcome <- data.frame(id = seq_len(n), y = rnorm(n))

  fit <- imr(
    platforms, outcome, type_outcome = "continuous",
    sample_mcmc = c(12, 6), ssize = 5, seed = 17
  )

  expect_equal(fit$model_bitstrings, "11")
  expect_equal(dim(fit$theta_sample[[1]]), c(12L, 0L))
  expect_equal(dim(fit$theta_sample[[2]]), c(12L, 0L))
})

test_that("cv_imr returns labelled accuracy matrices", {
  cv <- cv_imr(fit_bin, k = 5, rounds = 2)
  expect_named(cv, c("total_cindex", "subset_cindex"))
  expect_equal(ncol(cv$total_cindex), length(fit_bin$model_bitstrings) + 1L)
  expect_equal(nrow(cv$total_cindex), 2L)
  expect_equal(tail(colnames(cv$total_cindex), 1), "all")
  expect_identical(attr(cv, "metric"), "AUC")
  expect_true(all(is.finite(cv$total_cindex)))
})

test_that("the cross-validated metric reflects the outcome type", {
  fc <- fit_demo("continuous", total = 200, burn = 100, seed = 4)
  expect_identical(attr(cv_imr(fc, k = 5, rounds = 1), "metric"), "MSE")
  fs <- fit_demo("right.censored", total = 200, burn = 100, seed = 4)
  expect_identical(attr(cv_imr(fs, k = 5, rounds = 1), "metric"), "C-index")
})

test_that("survival cross-validated C-index beats chance", {
  fs <- fit_demo("right.censored", total = 800, burn = 300, seed = 8)
  cv <- cv_imr(fs, k = 5, rounds = 3)
  expect_gt(mean(cv$total_cindex[, "all"]), 0.6)
})

test_that("cv_imr validates cross-validation controls", {
  expect_error(cv_imr(fit_bin, k = 1), "`k`")
  expect_error(cv_imr(fit_bin, k = max(fit_bin$sample_size) + 1), "sample size")
  expect_error(cv_imr(fit_bin, rounds = 0), "`rounds`")
  expect_error(cv_imr(fit_bin, max_models = 0), "`max_models`")
  expect_error(cv_imr(fit_bin, method = "lasso"), "should be one of")
  expect_error(cv_imr(fit_bin, method = "BMS"), "must match the fitted object")
})

test_that("cv_imr accepts an explicitly matching fitted method", {
  fit_bms <- imr(
    simIMR$platforms, simIMR$outcome.binary, cov = simIMR$covariates,
    type_outcome = "binary", method = "BMS", nu = c(-4, -3, -4),
    sample_mcmc = c(80, 40), ssize = 30, seed = 14
  )
  cv <- cv_imr(fit_bms, k = 5, rounds = 1, method = "BMS")
  expect_identical(attr(cv, "metric"), "AUC")
  expect_true(all(vapply(fit_bms$theta_sample, is.null, logical(1))))
})

test_that("predict() returns one data frame per subgroup model", {
  new_x <- simIMR$platforms$genomic[1:20, ]
  new_p <- simIMR$platforms$proteomic[1:20, ]
  pr <- predict(fit_bin, newdata = list(new_x, new_p),
                platform_names = c("1", "2"),
                covariates = simIMR$covariates)
  expect_type(pr, "list")
  expect_equal(length(pr), length(fit_bin$model_bitstrings))
  for (d in pr) expect_true(all(c("id", "predict") %in% names(d)))
})

test_that("predict_imr() dispatches through the predict() S3 method", {
  new_x <- simIMR$platforms$genomic[1:20, ]
  new_p <- simIMR$platforms$proteomic[1:20, ]
  expect_equal(
    predict_imr(fit_bin, newdata = list(new_x, new_p),
                platform_names = c("1", "2"),
                covariates = simIMR$covariates),
    predict(fit_bin, newdata = list(new_x, new_p),
            platform_names = c("1", "2"),
            covariates = simIMR$covariates)
  )
})

test_that("platform_names defaults to the training order", {
  new_x <- simIMR$platforms$genomic[1:30, ]
  new_p <- simIMR$platforms$proteomic[1:30, ]
  a <- predict(fit_bin, newdata = list(new_x, new_p),
               covariates = simIMR$covariates)                       # default
  b <- predict(fit_bin, newdata = list(new_x, new_p),
               platform_names = c("1", "2"), covariates = simIMR$covariates)
  expect_equal(a, b)
})

test_that("binary predictions are probabilities in [0, 1]", {
  new_x <- simIMR$platforms$genomic[1:60, ]
  new_p <- simIMR$platforms$proteomic[1:60, ]
  pr <- predict(fit_bin, newdata = list(new_x, new_p),
                covariates = simIMR$covariates)
  vals <- unlist(lapply(pr, function(d) as.numeric(d$predict)))
  vals <- vals[is.finite(vals)]
  expect_gt(length(vals), 0)
  expect_true(all(vals >= 0 & vals <= 1))
})

test_that("subjects are routed to the correct availability subgroup", {
  # ids 1:30 are present in genomic (1..240) and proteomic (1..180) but not
  # metabolomic (121..300), so they belong to subgroup '011'.
  new_x <- simIMR$platforms$genomic[1:30, ]
  new_p <- simIMR$platforms$proteomic[1:30, ]
  pr <- predict(fit_bin, newdata = list(new_x, new_p),
                covariates = simIMR$covariates)
  expect_equal(nrow(pr[["model:011"]]), 30L)
  expect_equal(nrow(pr[["model:100"]]), 0L)
})

test_that("continuous predictions correlate with the truth", {
  fit <- fit_demo("continuous", total = 800, burn = 300, seed = 9)
  new_x <- simIMR$platforms$genomic[simIMR$platforms$genomic$id <= 120, ]
  new_p <- simIMR$platforms$proteomic[simIMR$platforms$proteomic$id <= 120, ]
  pr <- predict(fit, newdata = list(new_x, new_p),
                covariates = simIMR$covariates)
  d <- merge(pr[["model:011"]], simIMR$outcome.continuous, by = "id")
  expect_gt(stats::cor(d$predict, d$y), 0.5)
})

test_that("prediction is invariant to newdata and covariate row order", {
  new_x <- simIMR$platforms$genomic[1:60, ]
  new_p <- simIMR$platforms$proteomic[1:60, ]
  base <- predict(fit_bin, newdata = list(new_x, new_p),
                  covariates = simIMR$covariates)

  set.seed(101)
  shuffled <- predict(
    fit_bin,
    newdata = list(new_x[sample(nrow(new_x)), ],
                   new_p[sample(nrow(new_p)), ]),
    covariates = simIMR$covariates[sample(nrow(simIMR$covariates)), ]
  )

  sort_predictions <- function(x) {
    lapply(x, function(d) {
      d <- d[order(as.character(d$id)), , drop = FALSE]
      rownames(d) <- NULL
      d
    })
  }
  expect_equal(sort_predictions(shuffled), sort_predictions(base))
})

test_that("predict validates new data, covariates and model averaging controls", {
  new_x <- simIMR$platforms$genomic[1:10, ]
  expect_error(
    predict(fit_bin, newdata = list(new_x)),
    "`covariates` is required"
  )

  no_id <- new_x
  names(no_id)[1] <- "sample_id"
  expect_error(
    predict(fit_bin, newdata = list(no_id), covariates = simIMR$covariates),
    "id.*first column"
  )

  expect_error(
    predict(fit_bin, newdata = list(new_x), platform_names = "4",
            covariates = simIMR$covariates),
    "`platform_names`"
  )

  missing_feature <- new_x[, -2]
  expect_error(
    predict(fit_bin, newdata = list(missing_feature),
            covariates = simIMR$covariates),
    "feature column"
  )

  renamed_covariates <- simIMR$covariates
  names(renamed_covariates)[2] <- "age2"
  expect_error(
    predict(fit_bin, newdata = list(new_x), covariates = renamed_covariates),
    "covariate names"
  )

  expect_error(
    predict(fit_bin, newdata = list(new_x), covariates = simIMR$covariates,
            max_models = 0),
    "`max_models`"
  )
})

test_that("predict returns empty model outputs when no subjects can be routed", {
  new_x <- simIMR$platforms$genomic[1:5, ]
  new_x$id <- new_x$id + 10000
  res <- NULL
  expect_warning(
    res <- predict(fit_bin, newdata = list(new_x), covariates = simIMR$covariates),
    "No subjects"
  )
  expect_named(res, paste0("model:", fit_bin$model_bitstrings))
  expect_true(all(vapply(res, nrow, integer(1)) == 0L))
})

test_that("predict preserves character ids while keeping predictions numeric", {
  platforms <- lapply(simIMR$platforms, function(x) {
    x$id <- paste0("id", x$id)
    x
  })
  outcome <- simIMR$outcome.binary
  outcome$id <- paste0("id", outcome$id)
  covariates <- simIMR$covariates
  covariates$id <- paste0("id", covariates$id)

  fit <- imr(
    platforms, outcome, cov = covariates, type_outcome = "binary",
    nu = c(-4, -3, -4), sample_mcmc = c(80, 40), ssize = 30, seed = 12
  )
  pr <- predict(
    fit,
    newdata = list(platforms$genomic[1:8, ], platforms$proteomic[1:8, ]),
    covariates = covariates
  )

  expect_type(pr[["model:011"]]$id, "character")
  expect_type(pr[["model:011"]]$predict, "double")
})

test_that("predict works for fits without clinical covariates", {
  fit <- imr(
    simIMR$platforms, simIMR$outcome.continuous,
    type_outcome = "continuous", nu = c(-4, -3, -4),
    sample_mcmc = c(80, 40), ssize = 30, seed = 13
  )

  pr <- predict(
    fit,
    newdata = list(simIMR$platforms$genomic[1:12, ],
                   simIMR$platforms$proteomic[1:12, ])
  )
  expect_type(pr[["model:011"]]$predict, "double")
  expect_equal(nrow(pr[["model:011"]]), 12L)

  expect_warning(
    predict(
      fit,
      newdata = list(simIMR$platforms$genomic[1:3, ],
                     simIMR$platforms$proteomic[1:3, ]),
      covariates = simIMR$covariates
    ),
    "ignoring"
  )
})

test_that("predict uses the fitted non-local prior scale during model averaging", {
  set.seed(21)
  n <- 36
  platform <- data.frame(id = seq_len(n), x1 = rnorm(n), x2 = rnorm(n))
  outcome <- data.frame(id = seq_len(n), y = 3 * platform$x1 + rnorm(n, sd = 0.2))

  fit <- imr(
    list(platform = platform), outcome, type_outcome = "continuous",
    nu = 2, hh = 0.02, sample_mcmc = c(120, 60), ssize = 5, seed = 21
  )
  base <- predict(fit, newdata = list(platform[1:12, ]))[["model:1"]]$predict

  changed <- fit
  changed$list_hyperpara[2] <- 20
  shifted <- predict(changed, newdata = list(platform[1:12, ]))[["model:1"]]$predict

  expect_false(isTRUE(all.equal(base, shifted, tolerance = 1e-12)))
})

test_that("all three outcome types fit without error and return valid output", {
  for (type in c("binary", "continuous", "right.censored")) {
    fit <- fit_demo(type, total = 200, burn = 100, seed = 3)
    expect_s3_class(fit, "imr")
    expect_identical(fit$type_outcome, type)
    expect_true(all(is.finite(fit$log_posterior)))
    expect_true(all(vapply(coef(fit), function(m) all(m >= 0 & m <= 1), logical(1))))
  }
})

test_that("the BMS (non-integrative) method also runs", {
  fit <- imr(
    simIMR$platforms, simIMR$outcome.binary, cov = simIMR$covariates,
    type_outcome = "binary", method = "BMS",
    nu = c(-4, -3, -4), sample_mcmc = c(150, 75), ssize = 30, seed = 1)
  expect_s3_class(fit, "imr")
  expect_identical(fit$method, "BMS")
})

test_that("the model runs without clinical covariates", {
  fit <- imr(
    simIMR$platforms, simIMR$outcome.binary, cov = NULL,
    type_outcome = "binary",
    nu = c(-4, -3, -4), sample_mcmc = c(150, 75), ssize = 30, seed = 1)
  expect_s3_class(fit, "imr")
  expect_equal(fit$data1[[7]], 0)  # n_cov == 0
})

test_that("integer-valued outcomes are handled (coerced to double)", {
  oc <- simIMR$outcome.binary
  oc$y <- as.integer(oc$y)
  expect_silent(
    suppressWarnings(imr(
      simIMR$platforms, oc, cov = simIMR$covariates, type_outcome = "binary",
      nu = c(-4, -3, -4), sample_mcmc = c(120, 60), ssize = 30, seed = 1))
  )
})

#' Predict Outcomes for New Subjects
#'
#' @description
#' `predict()` method for objects of class `"imr"` produced by [imr()].  New
#' subjects are matched to the availability subgroup models learned during
#' training, missing platforms are handled automatically, and the
#' test features are standardized with the training set's centring and scaling
#' factors before Bayesian model averaging.
#'
#' @param object A fitted object of class `"imr"` returned by [imr()].
#' @param newdata A list of data frames with the new platform measurements.
#'   Each data frame must include `id` as the first column, followed by finite
#'   numeric feature columns matching the corresponding training platform.
#' @param platform_names A character vector giving, for each element of
#'   `newdata`, the index (`"1"`, `"2"`, ...) of the corresponding training
#'   platform.  Defaults to `NULL`, meaning the elements of `newdata` are taken
#'   to be in the same order as the platforms supplied to [imr()].
#' @param covariates An optional data frame of clinical covariates for the test
#'   subjects, including `id` as the first column.  Required when the model was
#'   fitted with covariates and ignored with a warning when it was not.
#' @param max_models Integer; the maximum number of distinct selection models
#'   (gamma configurations) used for Bayesian model averaging.  Default `100`.
#' @param verbose Logical; if `TRUE`, print the C routine's diagnostics.
#'   Defaults to `FALSE`.
#' @param ... Unused; present for S3 compatibility.
#'
#' @details
#' Predictions are only produced for subjects observed on at least one platform
#' (and, when `covariates` is supplied, with covariate data).  For `"binary"`
#' outcomes the returned `predict` column is a probability obtained through the
#' probit link (`pnorm`); for `"continuous"` and `"right.censored"` outcomes it
#' is the predicted (latent) response.
#'
#' @return A named list with one data frame per active availability subgroup
#'   model. Each data frame has columns `id` (subject identifier) and `predict`
#'   (predicted value or probability, rounded to three decimals).
#'
#' @seealso [imr()], [cv_imr()]
#'
#' @examples
#' \donttest{
#' data("simIMR", package = "IntegMultiReg")
#' fit <- imr(
#'   platform_data_list = simIMR$platforms, outcome = simIMR$outcome,
#'   cov = simIMR$covariates, type_outcome = "binary",
#'   nu = c(-4, -3, -4), sample_mcmc = c(200, 100), ssize = 5, seed = 1
#' )
#' new_x <- simIMR$platforms[[1]][1:10, ]
#' new_z <- simIMR$platforms[[2]][1:7, ]
#' predict(fit, newdata = list(new_x, new_z), covariates = simIMR$covariates)
#' }
#' @export
predict.imr <- function(object, newdata, platform_names = NULL,
                        covariates = NULL, max_models = 100,
                        verbose = FALSE, ...) {
  if (!inherits(object, "imr")) {
    .imr_abort("`object` must be an `imr` object returned by `imr()`.")
  }
  .imr_check_flag(verbose, "verbose")
  max_models <- .imr_check_integer_scalar(max_models, "max_models", min = 1)
  type_outcome <- object$type_outcome
  if (is.null(type_outcome)) {
    type_outcome <- "right.censored"
  }
  method <- object$method
  if (is.null(method)) method <- "IMR"
  n_platform <- as.integer(object$data1[[1]])
  n_cov <- as.integer(object$data1[[7]])

  if (!is.list(newdata) || length(newdata) == 0L) {
    .imr_abort("`newdata` must be a non-empty list of data frames.")
  }
  if (length(newdata) > n_platform) {
    .imr_abort("`newdata` cannot contain more platforms than the fitted model.")
  }

  ## Default: the new platforms are supplied in the same order as at training.
  if (is.null(platform_names)) {
    platform_names <- as.character(seq_along(newdata))
  } else {
    if (length(platform_names) != length(newdata)) {
      .imr_abort("`platform_names` must have the same length as `newdata`.")
    }
    platform_names <- as.character(platform_names)
    if (anyNA(platform_names) || anyDuplicated(platform_names)) {
      .imr_abort("`platform_names` must not contain missing or duplicated values.")
    }
    valid_platforms <- as.character(seq_len(n_platform))
    if (any(!platform_names %in% valid_platforms)) {
      .imr_abort(sprintf(
        "`platform_names` must use training platform indices in {%s}.",
        paste(valid_platforms, collapse = ", ")
      ))
    }
  }
  names(newdata) <- platform_names
  model_names <- names((object$data2[[1]])) ### binary indicators of platforms active in each subgroup

  for (i in seq_along(newdata)) {
    arg <- sprintf("newdata[[%d]]", i)
    platform_index <- as.integer(platform_names[i])
    .imr_check_id_frame(newdata[[i]], arg, require_rows = FALSE)
    .imr_check_numeric_columns(newdata[[i]], arg)
    expected_n <- object$data1[[6]][platform_index]
    if (ncol(newdata[[i]]) - 1L != expected_n) {
      .imr_abort(sprintf(
        "`%s` must contain %d feature column(s) for training platform %s.",
        arg, expected_n, platform_names[i]
      ))
    }
    expected_names <- object$feature_names[[platform_index]]
    if (!is.null(expected_names) &&
        !identical(colnames(newdata[[i]])[-1], expected_names)) {
      .imr_abort(sprintf(
        "Feature columns in `%s` must match the training feature names.",
        arg
      ))
    }
  }

  if (n_cov > 0L && is.null(covariates)) {
    .imr_abort("`covariates` is required because the model was fitted with covariates.")
  }
  if (n_cov == 0L && !is.null(covariates)) {
    .imr_warn(
      "`covariates` was supplied, but the model was fitted without covariates; ignoring it."
    )
    covariates <- NULL
  }
  if (!is.null(covariates)) {
    .imr_check_id_frame(covariates, "covariates", require_rows = FALSE)
    .imr_check_numeric_columns(covariates, "covariates")
    if (ncol(covariates) - 1L != n_cov) {
      .imr_abort(sprintf(
        "`covariates` must contain %d covariate column(s).", n_cov
      ))
    }
    expected_covariates <- object$covariate_names
    if (!is.null(expected_covariates) && length(expected_covariates) > 0L &&
        !identical(colnames(covariates)[-1], expected_covariates)) {
      .imr_abort("Columns in `covariates` must match the training covariate names.")
    }
  }

  all_ids <- unique(unlist(lapply(newdata, function(df) df$id)))

  ## We only predict subjects with covariate data that are observed in at least one omics platform
  if (!is.null(covariates)) {
    all_ids <- intersect(all_ids, covariates$id)
  }

  if (length(all_ids) == 0) {
    .imr_warn(
      "No subjects have both the required covariates and at least one platform."
    )
    return(.imr_empty_predictions(model_names))
  }
  # Rows correspond to subjects and columns correspond to platforms.
  presence <- data.frame(do.call(cbind, lapply(newdata, function(df) {
    all_ids %in% df$id
  })))
  names(presence) <- platform_names
  not_active_platform <- setdiff(as.character(1:n_platform), names(presence))
  for (l in not_active_platform) {
    presence[[l]] <- rep(FALSE, length(all_ids))
  }
  presence <- presence[, as.character(c(1:n_platform)), drop = FALSE]

  bitstrings <- apply(
    presence, 1,
    function(x) paste(as.integer(rev(x)), collapse = "")
  )

  x_train <- object$data2[[1]]
  unique_patterns <- model_names
  platforms <- newdata
  for (l in not_active_platform) {
    platforms[[l]] <- data.frame(matrix(nrow = 0, ncol = ncol(x_train[[1]][[as.numeric(l)]])))
  }

  platforms <- platforms[as.character(c(1:n_platform))]
  nplat <- n_platform
  x_test <- list()
  sample_ids <- list()

  for (pat in unique_patterns) {
    subgroup_ids <- all_ids[bitstrings == pat]
    subgroup_list <- vector("list", nplat)
    names(subgroup_list) <- paste0("platform", seq_len(nplat))
    for (i in seq_len(nplat)) {
      bit <- substr(pat, nplat - i + 1, nplat - i + 1)
      if (bit == "1") {
        subgroup_list[[i]] <- .imr_match_rows(
          platforms[[i]], subgroup_ids, sprintf("newdata platform %d", i)
        )
      } else {
        subgroup_list[[i]] <- platforms[[i]][FALSE, , drop = FALSE]
      }
    }
    sample_ids[[pat]] <- subgroup_ids
    x_test[[pat]] <- subgroup_list
  }

  if (length(unlist(sample_ids)) == 0) {
    .imr_warn(
      "No new subjects belong to availability subgroup models retained during training."
    )
    return(.imr_empty_predictions(model_names))
  }
  routed_ids <- unique(unlist(sample_ids, use.names = FALSE))
  dropped_ids <- setdiff(as.character(all_ids), as.character(routed_ids))
  if (length(dropped_ids) > 0L) {
    .imr_warn(sprintf(
      "%d subject(s) do not belong to availability subgroup models retained during training and were not predicted.",
      length(dropped_ids)
    ))
  }

  ### Create a vector that shows the presence of models in the test data
  samplesize_test <- unlist(lapply(sample_ids, length))
  if (!is.null(covariates)) {
    cova_test <- lapply(sample_ids, function(x) {
      .imr_match_rows(covariates, x, "covariates")
    })
  } else {
    cova_test <- lapply(sample_ids, function(x) matrix(numeric(0), nrow = length(x), ncol = 0))
  }
  x_test <- lapply(x_test, function(subgroup) {
    lapply(subgroup, function(platform_data) {
      if (is.data.frame(platform_data) && "id" %in% colnames(platform_data)) {
        mat <- as.matrix(platform_data[, -1, drop = FALSE])
      } else if (!is.null(colnames(platform_data)) &&
        colnames(platform_data)[1] == "id") {
        mat <- as.matrix(platform_data[, -1, drop = FALSE])
      } else {
        mat <- as.matrix(platform_data)
      }
      storage.mode(mat) <- "double"
      return(mat)
    })
  })

  if (!is.null(covariates)) {
    cova_test <- lapply(
      seq_along(cova_test),
      function(i) as.matrix(cova_test[[i]][, -1, drop = FALSE])
    )
  }

  norm_mean <- object$data2[[4]]
  norm_sd <- object$data2[[5]]

  x_test <- normalize_nested_list_known_mean_sd(x_test, norm_mean, norm_sd)

  mean_cov <- object$data2[[6]]
  sd_cov <- object$data2[[7]]

  if (!is.null(covariates)) {
    cova_test <- mapply(normalize_matrix_known_mean_variance,
      cova_test, mean_cov, sd_cov,
      SIMPLIFY = FALSE
    )
  }

  method_c <- as.character(method)
  results <- .quietly(verbose, .Call("mainFunctionPredictionTest",
    h0_c = object$list_hyperpara[1],
    hh_c = object$list_hyperpara[2],
    alpha_c = object$list_hyperpara[3],
    psi_c = object$list_hyperpara[4],
    alpha0_c = object$list_hyperpara[5],
    beta0_c = object$list_hyperpara[6],
    seed_c = object$list_hyperpara[7],
    nu_c = object$list_hyperpara[8:(7 + n_platform)],
    y_latent = object$estimate_latent_y,
    gam_sample_c = object$gam_sample,
    theta_c = object$theta_mean,
    method_c = method_c,
    n_platform_c = as.integer(n_platform),
    platform_models_c = object$data1[[2]],
    model_platforms_c = object$data1[[3]],
    n_models = as.integer(object$data1[[4]]),
    sample_size = as.integer(object$data1[[5]]),
    n_features = as.integer(object$data1[[6]]),
    n_cov = as.integer(object$data1[[7]]),
    x_filtered = x_train,
    cov_list = object$data2[[3]],
    ## data1[[9]] is the number of retained MCMC draws (= length(gam_sample));
    ## it indexes the draws used for Bayesian model averaging.
    sample = as.integer(object$data1[[9]]),
    x_test = x_test,
    c_test = cova_test,
    samplesize_test_c = as.integer(samplesize_test),
    max_models_pred = as.integer(max_models)
  ))
  names(results) <- model_names
  if (type_outcome == "binary") {
    res <- mapply(function(x, y) {
      data.frame(
        id = x,
        predict = round(pnorm(y), digits = 3),
        row.names = NULL,
        stringsAsFactors = FALSE
      )
    }, sample_ids, results, SIMPLIFY = FALSE)
  } else {
    res <- mapply(function(x, y) {
      data.frame(
        id = x,
        predict = round(y, digits = 3),
        row.names = NULL,
        stringsAsFactors = FALSE
      )
    }, sample_ids, results, SIMPLIFY = FALSE)
  }

  names(res) <- paste("model:", model_names, sep = "")
  return(res)
}

test_that("outcome dimension is validated per outcome type", {
  # binary/continuous need exactly 2 columns (id + response)
  bad <- data.frame(id = simIMR$outcome.binary$id,
                    y = simIMR$outcome.binary$y, extra = 1)
  expect_error(
    imr(simIMR$platforms, bad,
      type_outcome = "binary", sample_mcmc = c(50, 25), ssize = 30),
    "id"
  )
  # right.censored needs exactly 3 columns
  expect_error(
    imr(simIMR$platforms, simIMR$outcome.binary,
      type_outcome = "right.censored", sample_mcmc = c(50, 25), ssize = 30),
    "right-censored"
  )
})

test_that("retained-draw and burn-in counts are validated", {
  # the number of retained draws must be positive
  expect_error(
    imr(simIMR$platforms, simIMR$outcome.binary,
      type_outcome = "binary", sample_mcmc = c(0, 25), ssize = 30),
    "retained draws"
  )
  # a retained count below the burn-in is allowed (sample_mcmc = retained + burn-in)
  expect_s3_class(
    imr(simIMR$platforms, simIMR$outcome.binary,
      type_outcome = "binary", sample_mcmc = c(60, 80), ssize = 30, seed = 1),
    "imr"
  )
})

test_that("an ssize that excludes every subgroup is an error", {
  expect_error(
    imr(simIMR$platforms, simIMR$outcome.binary,
      type_outcome = "binary", sample_mcmc = c(50, 25), ssize = 1e6),
    "No availability subgroup"
  )
})

test_that("type_outcome and method are matched against their choices", {
  expect_error(
    imr(simIMR$platforms, simIMR$outcome.binary,
      type_outcome = "poisson", sample_mcmc = c(50, 25)),
    "should be one of"
  )
  expect_error(
    imr(simIMR$platforms, simIMR$outcome.binary,
      type_outcome = "binary", method = "lasso", sample_mcmc = c(50, 25)),
    "should be one of"
  )
})

test_that("downstream functions reject non-imr input", {
  expect_error(cv_imr(list(1)), "imr")
  expect_error(IntegMultiReg:::predict.imr(list(1), newdata = list()), "imr")
})

test_that("input data frames have standard id and numeric-column validation", {
  no_id <- simIMR$platforms
  names(no_id[[1]])[1] <- "sample_id"
  expect_error(
    imr(no_id, simIMR$outcome.binary, cov = simIMR$covariates,
        type_outcome = "binary", sample_mcmc = c(50, 25), ssize = 30),
    "id.*first column"
  )

  dup_id <- simIMR$platforms
  dup_id[[1]]$id[2] <- dup_id[[1]]$id[1]
  expect_error(
    imr(dup_id, simIMR$outcome.binary, cov = simIMR$covariates,
        type_outcome = "binary", sample_mcmc = c(50, 25), ssize = 30),
    "unique subject identifiers"
  )

  non_numeric <- simIMR$platforms
  non_numeric[[1]]$G01 <- as.character(non_numeric[[1]]$G01)
  expect_error(
    imr(non_numeric, simIMR$outcome.binary, cov = simIMR$covariates,
        type_outcome = "binary", sample_mcmc = c(50, 25), ssize = 30),
    "must be numeric"
  )

  bad_binary <- simIMR$outcome.binary
  bad_binary$y[1] <- 2
  expect_error(
    imr(simIMR$platforms, bad_binary, cov = simIMR$covariates,
        type_outcome = "binary", sample_mcmc = c(50, 25), ssize = 30),
    "0/1"
  )

  bad_survival <- simIMR$outcome.survival
  bad_survival$time[1] <- 0
  expect_error(
    imr(simIMR$platforms, bad_survival, cov = simIMR$covariates,
        type_outcome = "right.censored", sample_mcmc = c(50, 25), ssize = 30),
    "positive"
  )
})

test_that("scalar and hyper-parameter arguments are validated before sampling", {
  expect_error(
    imr(simIMR$platforms, simIMR$outcome.binary,
        type_outcome = "binary", nu = c(-3, -3), sample_mcmc = c(50, 25)),
    "`nu`"
  )
  expect_error(
    imr(simIMR$platforms, simIMR$outcome.binary,
        type_outcome = "binary", hh = -1, sample_mcmc = c(50, 25)),
    "`hh`"
  )
  expect_error(
    imr(simIMR$platforms, simIMR$outcome.binary,
        type_outcome = "binary", sample_mcmc = c(50.5, 25)),
    "`sample_mcmc`"
  )
  expect_error(
    imr(simIMR$platforms, simIMR$outcome.binary,
        type_outcome = "binary", ssize = -1, sample_mcmc = c(50, 25)),
    "`ssize`"
  )
  expect_error(
    imr(simIMR$platforms, simIMR$outcome.binary,
        type_outcome = "binary", verbose = NA, sample_mcmc = c(50, 25)),
    "`verbose`"
  )
})

## Stand-alone plotting helpers for objects of class "imr".

#' Plot the Top Selected Features of an IMR Fit
#'
#' @description
#' Draws a horizontal bar chart of the features with the highest marginal
#' posterior inclusion probability (mPIP).  For each platform-feature pair, the
#' plotted score is the highest mPIP attained among the availability subgroups
#' containing that platform; the displayed bars are the largest such scores
#' across all platforms.  This complements the per-platform heatmap of
#' `plot(fit, type = "selection")`.
#'
#' @param object A fitted object of class `"imr"` returned by [imr()].
#' @param top Integer; the number of highest-mPIP features to display
#'   (default `10`).
#' @param base_cex Overall text-size multiplier. The `cex_*` arguments default
#'   to values derived from this multiplier (default `1`).
#' @param cex_names Feature-label size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param cex_axis Axis-label size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param cex_lab Axis-title size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param cex_main Main-title size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param cex_legend Legend text size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param col Optional bar colours. If `NULL`, colours are generated from
#'   `palette`.
#' @param palette Palette name used when `col = NULL`; `"platform"` gives the
#'   package's standard muted platform colours (default).
#' @param reference Optional vertical reference line; use `NULL` to suppress it
#'   (default `0.5`).
#' @param show_source Logical; should each bar be annotated with the
#'   availability subgroup (bitstring) in which the feature attained its maximum
#'   mPIP (default `TRUE`)?
#' @param legend Logical; should the platform legend be drawn (default `TRUE`)?
#' @param xlim Numeric vector of length two giving the horizontal axis limits
#'   (default `c(0, 1)`).
#' @param mar,mgp Optional graphical margin and axis-title placement vectors
#'   passed to [graphics::par()] for finer layout control.
#' @param ... Further graphical parameters passed to [graphics::barplot()].
#'
#' @return Invisibly, a data frame of the displayed features with columns
#'   `platform`, `feature`, `mpip` and `subgroup`, where `mpip` is the maximum
#'   subgroup mPIP for that platform-feature pair and `subgroup` is the
#'   subgroup where the maximum is attained. Rows are ordered by decreasing
#'   mPIP.
#' @seealso [imr()], [plot.imr()], [plot_subgroup_sizes()]
#' @examples
#' \donttest{
#' data("simIMR", package = "IntegMultiReg")
#' fit <- imr(
#'   platform_data_list = simIMR$platforms, outcome = simIMR$outcome,
#'   cov = simIMR$covariates, type_outcome = "binary",
#'   nu = c(-4, -3, -4), sample_mcmc = c(200, 100), ssize = 5, seed = 1
#' )
#' plot_top_features(fit, top = 8)
#' }
#' @export
plot_top_features <- function(object, top = 10, base_cex = 1,
                              cex_names = NULL, cex_axis = NULL,
                              cex_lab = NULL, cex_main = NULL,
                              cex_legend = NULL, col = NULL,
                              palette = "platform", reference = 0.5,
                              show_source = TRUE, legend = TRUE, xlim = c(0, 1),
                              mar = NULL, mgp = NULL, ...) {
  if (!inherits(object, "imr")) {
    .imr_abort("`object` must be an `imr` object returned by `imr()`.")
  }
  dots <- list(...)
  top <- .imr_check_integer_scalar(top, "top", min = 1)
  sz <- .imr_plot_cex(
    dots, base_cex = base_cex, cex_names = cex_names,
    cex_axis = cex_axis, cex_lab = cex_lab, cex_main = cex_main,
    cex_legend = cex_legend,
    defaults = list(axis = 0.95, lab = 1.05, main = 1.1,
                    names = 0.95, legend = 0.9, values = 0.9)
  )
  dots <- sz$dots
  .imr_check_flag(show_source, "show_source")
  .imr_check_flag(legend, "legend")
  xlim <- .imr_check_numeric_vector(xlim, "xlim", length = 2)
  if (xlim[1] >= xlim[2]) {
    .imr_abort("`xlim` must be increasing.")
  }
  if (!is.null(reference)) {
    reference <- .imr_check_numeric_vector(
      reference, "reference", length = 1, nonnegative = TRUE
    )
    if (reference > 1) {
      .imr_abort("`reference` must be between 0 and 1, or NULL.")
    }
  }
  ## Collect every feature's best mPIP (and the subgroup achieving it).
  rows <- list()
  for (l in seq_len(object$n_platform)) {
    m <- .imr_mpip(object, l)
    if (nrow(m) == 0 || ncol(m) == 0) next
    maxp <- apply(m, 2, max)
    sg <- rownames(m)[apply(m, 2, which.max)]
    rows[[l]] <- data.frame(
      platform = object$platform_names[l],
      feature = colnames(m),
      mpip = as.numeric(maxp),
      subgroup = sg,
      stringsAsFactors = FALSE
    )
  }
  tab <- do.call(rbind, rows)
  if (is.null(tab) || nrow(tab) == 0) {
    .imr_abort("The fit contains no selectable features to plot.")
  }
  tab <- tab[order(tab$mpip, decreasing = TRUE), , drop = FALSE]
  top <- min(top, nrow(tab))
  tab <- tab[seq_len(top), , drop = FALSE]

  ## One colour per platform, drawn in increasing order so the largest bar is
  ## at the top of the horizontal chart.
  platforms <- object$platform_names
  pal <- .imr_plot_palette(length(platforms), col = col, palette = palette)
  ord <- rev(seq_len(top))
  labels <- paste0(tab$feature[ord], " (", tab$platform[ord], ")")

  op <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(op), add = TRUE)
  pp <- .imr_plot_par(mar, mgp, default_mar = c(4.8, 8.8, 3.2, 1))
  graphics::par(mar = pp$mar, mgp = pp$mgp)
  if (is.null(dots$border)) dots$border <- NA
  bp <- do.call(graphics::barplot, c(list(
    height = tab$mpip[ord], names.arg = labels, horiz = TRUE, las = 1,
    xlim = xlim, xlab = "Maximum subgroup mPIP",
    main = sprintf("Top %d features by max subgroup mPIP", top),
    col = pal[match(tab$platform[ord], platforms)],
    cex.names = sz$names, cex.axis = sz$axis,
    cex.lab = sz$lab, cex.main = sz$main
  ), dots))
  if (!is.null(reference)) {
    graphics::abline(v = reference, lty = 2,
                     col = .imr_plot_reference_colour())
  }
  ## Annotate each bar with the availability subgroup where its maximum mPIP was
  ## attained.  Long bars carry the label inside the coloured region (light
  ## text); short bars carry it just past the tip (dark text) so it never
  ## overplots the reference line or spills past the axis.
  if (show_source) {
    xend <- tab$mpip[ord]
    src <- tab$subgroup[ord]
    inside <- xend >= xlim[1] + 0.5 * (xlim[2] - xlim[1])
    if (any(inside)) {
      graphics::text(xend[inside], bp[inside], labels = src[inside],
                     pos = 2, offset = 0.35, col = "white",
                     font = 2, cex = sz$values)
    }
    if (any(!inside)) {
      graphics::text(xend[!inside], bp[!inside], labels = src[!inside],
                     pos = 4, offset = 0.35,
                     col = .imr_plot_reference_colour(),
                     font = 2, cex = sz$values)
    }
  }
  if (legend) {
    graphics::legend("bottomright", legend = platforms, fill = pal,
                     bty = "n", cex = sz$legend)
  }
  invisible(tab[, c("platform", "feature", "mpip", "subgroup")])
}


#' Plot the Availability Subgroup Sizes of an IMR Fit
#'
#' @description
#' Draws a bar chart of the number of subjects in each modelled
#' availability subgroup (the non-empty regions of the Venn diagram), giving a
#' quick picture of how the sample is distributed across subgroups.
#'
#' @param object A fitted object of class `"imr"` returned by [imr()].
#' @param base_cex Overall text-size multiplier. The `cex_*` arguments default
#'   to values derived from this multiplier (default `1`).
#' @param cex_axis Axis-label size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param cex_lab Axis-title size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param cex_main Main-title size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param cex_values Bar-value label size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param col Optional bar colours. If `NULL`, colours are generated from
#'   `palette`.
#' @param palette Palette name used when `col = NULL`; `"subgroup"` gives the
#'   package's standard neutral grey-blue subgroup colours (default).
#' @param show_values Logical; should sample sizes be printed above the bars
#'   (default `TRUE`)?
#' @param ylim Optional numeric vector of length two giving the vertical axis
#'   limits. If `NULL`, limits are chosen from the subgroup sizes.
#' @param mar,mgp Optional graphical margin and axis-title placement vectors
#'   passed to [graphics::par()] for finer layout control.
#' @param ... Further graphical parameters passed to [graphics::barplot()].
#'
#' @return Invisibly, the named integer vector of subgroup sizes.
#' @seealso [imr()], [plot.imr()], [plot_top_features()]
#' @examples
#' \donttest{
#' data("simIMR", package = "IntegMultiReg")
#' fit <- imr(
#'   platform_data_list = simIMR$platforms, outcome = simIMR$outcome,
#'   cov = simIMR$covariates, type_outcome = "binary",
#'   nu = c(-4, -3, -4), sample_mcmc = c(200, 100), ssize = 5, seed = 1
#' )
#' plot_subgroup_sizes(fit)
#' }
#' @export
plot_subgroup_sizes <- function(object, base_cex = 1, cex_axis = NULL,
                                cex_lab = NULL, cex_main = NULL,
                                cex_values = NULL, col = NULL,
                                palette = "subgroup", show_values = TRUE,
                                ylim = NULL, mar = NULL, mgp = NULL, ...) {
  if (!inherits(object, "imr")) {
    .imr_abort("`object` must be an `imr` object returned by `imr()`.")
  }
  dots <- list(...)
  sz <- .imr_plot_cex(
    dots, base_cex = base_cex, cex_axis = cex_axis,
    cex_lab = cex_lab, cex_main = cex_main, cex_values = cex_values,
    defaults = list(axis = 0.95, lab = 1.05, main = 1.1,
                    names = 1, legend = 0.9, values = 0.95)
  )
  dots <- sz$dots
  .imr_check_flag(show_values, "show_values")
  sizes <- as.integer(object$sample_size)
  names(sizes) <- object$model_bitstrings
  if (is.null(ylim)) {
    ylim <- c(0, max(sizes) * 1.15)
  } else {
    ylim <- .imr_check_numeric_vector(ylim, "ylim", length = 2)
    if (ylim[1] >= ylim[2]) {
      .imr_abort("`ylim` must be increasing.")
    }
  }

  op <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(op), add = TRUE)
  pp <- .imr_plot_par(mar, mgp, default_mar = c(4.8, 4.8, 3.2, 1))
  graphics::par(mar = pp$mar, mgp = pp$mgp)
  if (is.null(dots$border)) dots$border <- NA
  bp <- do.call(graphics::barplot, c(list(
    height = sizes, xlab = "Availability subgroup (bitstring)",
    ylab = "Number of subjects", main = "Subjects per availability subgroup",
    col = .imr_plot_palette(length(sizes), col = col, palette = palette),
    ylim = ylim, cex.axis = sz$axis,
    cex.lab = sz$lab, cex.main = sz$main
  ), dots))
  if (show_values) {
    graphics::text(bp, sizes, labels = sizes, pos = 3, cex = sz$values)
  }
  invisible(sizes)
}

test_that("plot_top_features draws and returns the ranked feature table", {
  tmp <- tempfile(fileext = ".pdf")
  pdf(tmp)
  on.exit({ dev.off(); unlink(tmp) }, add = TRUE)

  tab <- plot_top_features(fit_bin, top = 8)
  expect_s3_class(tab, "data.frame")
  expect_equal(nrow(tab), 8L)
  expect_named(tab, c("platform", "feature", "mpip", "subgroup"))
  # returned table is sorted by decreasing mPIP and the values are valid
  expect_true(all(diff(tab$mpip) <= 0))
  expect_true(all(tab$mpip >= 0 & tab$mpip <= 1))
  expect_true(all(tab$platform %in% fit_bin$platform_names))
})

test_that("plot_top_features caps 'top' at the number of features", {
  tmp <- tempfile(fileext = ".pdf")
  pdf(tmp)
  on.exit({ dev.off(); unlink(tmp) }, add = TRUE)
  total_features <- sum(vapply(coef(fit_bin), ncol, integer(1)))
  tab <- plot_top_features(fit_bin, top = 1000)
  expect_equal(nrow(tab), total_features)
})

test_that("plot_subgroup_sizes draws and returns the subgroup sizes", {
  tmp <- tempfile(fileext = ".pdf")
  pdf(tmp)
  on.exit({ dev.off(); unlink(tmp) }, add = TRUE)

  sizes <- plot_subgroup_sizes(fit_bin)
  expect_equal(sizes[c("011", "100", "101", "111")],
               stats::setNames(c(120L, 60L, 60L, 60L),
                               c("011", "100", "101", "111")))
})

test_that("plot font-size controls accept underscore and base-R aliases", {
  tmp <- tempfile(fileext = ".pdf")
  pdf(tmp)
  on.exit({ dev.off(); unlink(tmp) }, add = TRUE)

  expect_silent(plot(fit_bin, type = "selection",
                     cex_axis = 1.1, cex_lab = 1.1, cex_main = 1.1))
  expect_silent(plot(fit_bin, type = "selection", base_cex = 0.9,
                     legend = FALSE, col = c("white", "grey80", "black"),
                     mar = c(5, 5, 3, 1), mgp = c(3, 1, 0)))
  expect_silent(plot(fit_bin, type = "theta", platform = 1,
                     cex.axis = 0.9, cex.lab = 0.9, cex.main = 0.9))
  expect_silent(plot(fit_bin, type = "theta", platform = 1,
                     palette = "heatmap"))
  expect_silent(plot(fit_bin, type = "trace", cex_axis = 0.9, col = "grey20"))

  expect_silent(plot_top_features(
    fit_bin, top = 5, cex_names = 0.9, cex_axis = 0.9,
    cex_lab = 0.9, cex_main = 0.9, cex_legend = 0.9
  ))
  expect_silent(plot_top_features(
    fit_bin, top = 5, cex.names = 0.9, cex.axis = 0.9,
    cex.lab = 0.9, cex.main = 0.9, cex.legend = 0.9
  ))
  expect_silent(plot_top_features(
    fit_bin, top = 5, base_cex = 0.9, legend = FALSE, reference = NULL,
    col = c("#4E79A7", "#59A14F", "#E15759"), xlim = c(0, 1)
  ))
  expect_silent(plot_top_features(fit_bin, top = 5, show_source = FALSE))
  expect_silent(plot_top_features(
    fit_bin, top = 5, palette = "platform"
  ))

  expect_silent(plot_subgroup_sizes(
    fit_bin, cex_axis = 0.9, cex_lab = 0.9,
    cex_main = 0.9, cex_values = 0.9
  ))
  expect_silent(plot_subgroup_sizes(
    fit_bin, cex.axis = 0.9, cex.lab = 0.9,
    cex.main = 0.9, cex.values = 0.9
  ))
  expect_silent(plot_subgroup_sizes(
    fit_bin, base_cex = 0.9, show_values = FALSE,
    col = "#4E79A7", ylim = c(0, 150)
  ))
  expect_silent(plot_subgroup_sizes(
    fit_bin, palette = "subgroup"
  ))
})

test_that("the stand-alone plot helpers reject non-imr input", {
  expect_error(plot_top_features(list(1)), "imr")
  expect_error(plot_subgroup_sizes(list(1)), "imr")
})

test_that("plot helpers validate user-facing controls", {
  expect_error(plot_top_features(fit_bin, top = 0), "`top`")
  expect_error(plot_top_features(fit_bin, cex_axis = 0), "`cex_axis`")
  expect_error(plot_top_features(fit_bin, base_cex = 0), "`base_cex`")
  expect_error(plot_top_features(fit_bin, reference = 2), "`reference`")
  expect_error(plot_top_features(fit_bin, legend = NA), "`legend`")
  expect_error(plot_top_features(fit_bin, show_source = NA), "`show_source`")
  expect_error(plot_top_features(fit_bin, xlim = c(1, 0)), "`xlim`")
  expect_error(plot_subgroup_sizes(fit_bin, cex_values = -1), "`cex_values`")
  expect_error(plot_subgroup_sizes(fit_bin, show_values = NA), "`show_values`")
  expect_error(plot_subgroup_sizes(fit_bin, ylim = c(1, 0)), "`ylim`")
  expect_error(plot(fit_bin, type = "selection", platform = 99), "`platform`")
  expect_error(plot(fit_bin, type = "selection", legend_width = 0),
               "`legend_width`")
})

## Internal helpers.

#' @keywords internal
#' @noRd
.imr_abort <- function(message) {
  stop(message, call. = FALSE)
}

#' @keywords internal
#' @noRd
.imr_warn <- function(message) {
  warning(message, call. = FALSE)
}

#' @keywords internal
#' @noRd
.imr_is_integerish <- function(x) {
  is.numeric(x) && all(is.finite(x)) && all(x == as.integer(x))
}

#' @keywords internal
#' @noRd
.imr_check_flag <- function(x, arg) {
  if (!is.logical(x) || length(x) != 1L || is.na(x)) {
    .imr_abort(sprintf("`%s` must be TRUE or FALSE.", arg))
  }
  invisible(x)
}

#' @keywords internal
#' @noRd
.imr_check_integer_scalar <- function(x, arg, min = -Inf, max = Inf) {
  if (length(x) != 1L || !.imr_is_integerish(x) || x < min || x > max) {
    range <- if (is.finite(min) && is.finite(max)) {
      sprintf(" between %s and %s", min, max)
    } else if (is.finite(min)) {
      sprintf(" greater than or equal to %s", min)
    } else if (is.finite(max)) {
      sprintf(" less than or equal to %s", max)
    } else {
      ""
    }
    .imr_abort(sprintf("`%s` must be a whole number%s.", arg, range))
  }
  as.integer(x)
}

#' @keywords internal
#' @noRd
.imr_check_integer_vector <- function(x, arg, length, min = -Inf) {
  if (!is.numeric(x) || length(x) != length || !.imr_is_integerish(x) ||
      any(x < min)) {
    .imr_abort(sprintf(
      "`%s` must be a numeric vector of %d whole number(s).",
      arg, length
    ))
  }
  as.integer(x)
}

#' @keywords internal
#' @noRd
.imr_check_numeric_vector <- function(x, arg, length = NULL,
                                      positive = FALSE,
                                      nonnegative = FALSE) {
  if (!is.numeric(x) || any(!is.finite(x)) ||
      (!is.null(length) && length(x) != length) ||
      (positive && any(x <= 0)) ||
      (nonnegative && any(x < 0))) {
    suffix <- if (!is.null(length)) {
      sprintf(" of length %d", length)
    } else {
      ""
    }
    bound <- if (positive) {
      " positive"
    } else if (nonnegative) {
      " non-negative"
    } else {
      ""
    }
    .imr_abort(sprintf("`%s` must be a finite%s numeric vector%s.",
                       arg, bound, suffix))
  }
  x
}

#' @keywords internal
#' @noRd
.imr_check_id_frame <- function(x, arg, require_rows = TRUE,
                                require_features = TRUE) {
  if (!is.data.frame(x)) {
    .imr_abort(sprintf("`%s` must be a data frame.", arg))
  }
  if (!identical(names(x)[1], "id")) {
    .imr_abort(sprintf("`%s` must have `id` as its first column.", arg))
  }
  if (require_rows && nrow(x) == 0L) {
    .imr_abort(sprintf("`%s` must contain at least one row.", arg))
  }
  if (require_features && ncol(x) < 2L) {
    .imr_abort(sprintf("`%s` must contain at least one non-id column.", arg))
  }
  if (anyNA(x$id)) {
    .imr_abort(sprintf("`%s$id` must not contain missing values.", arg))
  }
  if (anyDuplicated(x$id)) {
    .imr_abort(sprintf("`%s$id` must contain unique subject identifiers.", arg))
  }
  invisible(x)
}

#' @keywords internal
#' @noRd
.imr_check_numeric_columns <- function(x, arg, columns = names(x)[-1]) {
  if (length(columns) == 0L) {
    return(invisible(x))
  }
  bad_type <- columns[!vapply(x[columns], is.numeric, logical(1))]
  if (length(bad_type) > 0L) {
    .imr_abort(sprintf(
      "All non-id columns in `%s` must be numeric; problem column: `%s`.",
      arg, bad_type[1]
    ))
  }
  vals <- as.matrix(x[columns])
  if (any(!is.finite(vals))) {
    .imr_abort(sprintf("All non-id values in `%s` must be finite.", arg))
  }
  invisible(x)
}

#' @keywords internal
#' @noRd
.imr_match_rows <- function(x, ids, arg) {
  rows <- match(ids, x$id)
  if (anyNA(rows)) {
    .imr_abort(sprintf("`%s` is missing rows for one or more subject ids.", arg))
  }
  x[rows, , drop = FALSE]
}

#' @keywords internal
#' @noRd
.imr_empty_predictions <- function(model_names) {
  out <- lapply(model_names, function(x) {
    data.frame(id = character(0), predict = numeric(0))
  })
  names(out) <- paste0("model:", model_names)
  out
}

## Evaluate `code` while optionally discarding everything it writes to the
## console (including Rprintf output from the compiled sampler).  When
## `verbose` is TRUE the output is shown; otherwise it is captured and dropped.
#' @keywords internal
#' @noRd
.quietly <- function(verbose, code) {
  if (isTRUE(verbose)) {
    return(eval.parent(substitute(code)))
  }
  expr <- substitute(code)
  env <- parent.frame()
  utils::capture.output(value <- eval(expr, env))
  value
}

## Snake-case wrappers for the exported S3 methods.
##
## The S3 methods themselves must keep names such as predict.imr() and
## summary.imr() so that R can dispatch predict(fit), summary(fit), and friends.
## These wrappers give users a package-style snake_case API without bypassing
## that dispatch mechanism.

#' @rdname predict.imr
#' @export
predict_imr <- function(object, newdata, ...) {
  stats::predict(object, newdata = newdata, ...)
}


#' @rdname summary.imr
#' @export
summary_imr <- function(object, ...) {
  base::summary(object, ...)
}


#' @rdname coef.imr
#' @export
coef_imr <- function(object, ...) {
  stats::coef(object, ...)
}


#' @rdname plot.imr
#' @export
plot_imr <- function(x, ...) {
  graphics::plot(x, ...)
}

test_that("print returns its argument invisibly and prints key fields", {
  out <- capture.output(res <- print(fit_bin))
  expect_identical(res, fit_bin)
  expect_true(any(grepl("Outcome type", out)))
  expect_true(any(grepl("Platform key", out)))
  expect_true(any(grepl("P1 = genomic", out, fixed = TRUE)))
  expect_true(any(grepl("Availability subgroups modelled", out)))
  expect_true(any(grepl("011 : P1 + P2", out, fixed = TRUE)))
})

test_that("summary produces a per-platform selection table", {
  s <- summary(fit_bin, threshold = 0.5)
  expect_s3_class(s, "summary.imr")
  expect_named(s$selected, c("genomic", "proteomic", "metabolomic"))
  for (df in s$selected) {
    expect_true(all(c("feature", "max_mpip", "subgroup") %in% names(df)))
    # Selection uses the unrounded max mPIP, but the reported max_mpip is
    # rounded to 3 dp, so a feature just above the threshold displays as 0.500.
    if (nrow(df) > 0) expect_true(all(df$max_mpip >= 0.5))
  }
  expect_output(print(s), "summary")
})

test_that("snake-case summary and coef wrappers dispatch through S3", {
  expect_equal(summary_imr(fit_bin, threshold = 0.5),
               summary(fit_bin, threshold = 0.5))
  expect_equal(coef_imr(fit_bin), coef(fit_bin))
})

test_that("coef returns named per-platform mPIP matrices", {
  mp <- coef(fit_bin)
  expect_type(mp, "list")
  expect_named(mp, c("genomic", "proteomic", "metabolomic"))
  expect_equal(rownames(mp$genomic),
               fit_bin$model_bitstrings[fit_bin$platform_models[[1]]])
})

test_that("plot methods run without error for each type", {
  tmp <- tempfile(fileext = ".pdf")
  pdf(tmp)
  on.exit({ dev.off(); unlink(tmp) }, add = TRUE)
  expect_silent(plot(fit_bin, type = "selection"))
  expect_silent(plot(fit_bin, type = "theta"))
  expect_silent(plot(fit_bin, type = "trace"))
  expect_silent(plot(fit_bin, type = "selection", platform = 1))
  expect_silent(plot_imr(fit_bin, type = "selection", platform = 1))
})

## S3 methods for objects of class "imr".

## Internal: marginal posterior inclusion probability (mPIP) matrix for one
## platform, with subgroup (row) and feature (column) names attached.
#' @keywords internal
#' @noRd
.imr_mpip <- function(object, platform) {
  m <- object$gam_mean[[platform]]
  rn <- object$model_bitstrings[object$platform_models[[platform]]]
  cn <- object$feature_names[[platform]]
  if (!is.null(rn) && length(rn) == nrow(m)) rownames(m) <- rn
  if (!is.null(cn) && length(cn) == ncol(m)) colnames(m) <- cn
  m
}

## Internal: generic platform labels used to decode availability bitstrings.
#' @keywords internal
#' @noRd
.imr_platform_codes <- function(n_platform) {
  paste0("P", seq_len(n_platform))
}


## Internal: translate a bitstring into generic platform labels.  The right-most
## bit corresponds to P1, matching the convention used throughout the package.
#' @keywords internal
#' @noRd
.imr_bitstring_codes <- function(bitstring, n_platform) {
  bits <- strsplit(as.character(bitstring), "", fixed = TRUE)[[1]]
  if (length(bits) < n_platform) {
    bits <- c(rep("0", n_platform - length(bits)), bits)
  }
  present <- which(rev(bits) == "1")
  if (!length(present)) return("none")
  paste(.imr_platform_codes(n_platform)[present], collapse = " + ")
}


#' Marginal Posterior Inclusion Probabilities of an IMR Fit
#'
#' Extracts the posterior mean variable-selection probabilities (the marginal
#' posterior inclusion probabilities, mPIP) of a fitted model.
#'
#' @param object A fitted object of class `"imr"`.
#' @param ... Unused; present for S3 compatibility.
#' @return A named list with one matrix per platform.  Rows are the subgroups
#'   containing that platform (labelled by their availability bitstrings) and
#'   columns are the platform features.
#' @seealso [imr()]
#' @export
coef.imr <- function(object, ...) {
  out <- lapply(seq_len(object$n_platform), function(l) .imr_mpip(object, l))
  names(out) <- object$platform_names
  out
}


#' Print Method for IMR Fits
#'
#' Prints a compact overview of the fit, including a platform key (`P1`,
#' `P2`, ...) that decodes the availability-subgroup bitstrings.  Optionally,
#' it can also print the top-ranked features per platform, ranked by each
#' feature's maximum mPIP over availability subgroups containing that platform.
#'
#' @param x A fitted object of class `"imr"`.
#' @param threshold Inclusion-probability threshold used to count selected
#'   features (default `0.5`).
#' @param rank Logical; if `TRUE`, print a short ranked feature table for each
#'   platform (default `FALSE`).
#' @param top Integer; when `rank = TRUE`, the number of top-ranked features to
#'   show per platform (default `5`).
#' @param ... Unused; present for S3 compatibility.
#' @return `x`, invisibly.
#' @export
print.imr <- function(x, threshold = 0.5, rank = FALSE, top = 5, ...) {
  threshold <- .imr_check_numeric_vector(
    threshold, "threshold", length = 1, nonnegative = TRUE
  )
  if (threshold > 1) {
    .imr_abort("`threshold` must be between 0 and 1.")
  }
  .imr_check_flag(rank, "rank")
  top <- .imr_check_integer_scalar(top, "top", min = 1)
  cat("Integrative Bayesian Multi-Platform Regression (IMR)\n")
  cat("----------------------------------------------------\n")
  if (!is.null(x$call)) {
    cat("Call:\n  ")
    print(x$call)
  }
  cat(sprintf("\nOutcome type : %s\n", x$type_outcome))
  cat(sprintf("Method       : %s\n", x$method))
  cat(sprintf("Platforms    : %d (%s)\n", x$n_platform,
              paste(x$platform_names, collapse = ", ")))
  cat(sprintf("MCMC         : %d retained draws after %d burn-in\n",
              x$sample_mcmc[["total"]], x$sample_mcmc[["burnin"]]))

  codes <- .imr_platform_codes(x$n_platform)
  cat("\nPlatform key:\n")
  key <- paste(sprintf("  %s = %s", codes, x$platform_names), collapse = "\n")
  cat(key, "\n", sep = "")

  cat("\nAvailability subgroups modelled (bitstring : platforms : size):\n")
  subgroup_codes <- vapply(
    x$model_bitstrings, .imr_bitstring_codes, character(1),
    n_platform = x$n_platform
  )
  st <- paste(sprintf("  %-*s : %-*s : %d",
                      max(nchar(x$model_bitstrings)), x$model_bitstrings,
                      max(nchar(subgroup_codes)), subgroup_codes,
                      as.integer(x$sample_size)),
              collapse = "\n")
  cat(st, "\n", sep = "")

  cat(sprintf("\nFeatures with mPIP > %.2f (in any subgroup):\n", threshold))
  for (l in seq_len(x$n_platform)) {
    m <- x$gam_mean[[l]]
    sel <- if (nrow(m) > 0 && ncol(m) > 0) sum(apply(m, 2, max) > threshold) else 0L
    cat(sprintf("  %-12s : %d of %d\n", x$platform_names[l], sel, ncol(m)))
  }
  if (rank) {
    cat(sprintf("\nTop %d ranked features by maximum subgroup mPIP:\n", top))
    for (l in seq_len(x$n_platform)) {
      m <- .imr_mpip(x, l)
      cat(sprintf("  %s\n", x$platform_names[l]))
      if (nrow(m) == 0 || ncol(m) == 0) {
        cat("    (no selectable features)\n")
        next
      }
      maxp <- apply(m, 2, max)
      which_sg <- rownames(m)[apply(m, 2, which.max)]
      ord <- order(maxp, decreasing = TRUE)
      ord <- ord[seq_len(min(top, length(ord)))]
      tab <- data.frame(
        feature = colnames(m)[ord],
        max_mpip = round(maxp[ord], 3),
        subgroup = which_sg[ord],
        row.names = NULL,
        stringsAsFactors = FALSE
      )
      lines <- utils::capture.output(print(tab, row.names = FALSE))
      cat(paste0("    ", lines), sep = "\n")
      cat("\n")
    }
  }
  invisible(x)
}


#' Summarize an IMR Fit
#'
#' Produces a per-platform summary of the selected features (those whose
#' marginal posterior inclusion probability exceeds `threshold` in at least one
#' subgroup), ranked by their maximum inclusion probability.
#'
#' @param object A fitted object of class `"imr"`.
#' @param threshold Inclusion-probability threshold for selection (default
#'   `0.5`).
#' @param ... Unused; present for S3 compatibility.
#' @return An object of class `"summary.imr"`: a list with the run metadata and,
#'   for each platform, a data frame of selected features with their maximum
#'   mPIP and the subgroup achieving it.
#' @export
summary.imr <- function(object, threshold = 0.5, ...) {
  if (!inherits(object, "imr")) {
    .imr_abort("`object` must be an `imr` object returned by `imr()`.")
  }
  threshold <- .imr_check_numeric_vector(
    threshold, "threshold", length = 1, nonnegative = TRUE
  )
  if (threshold > 1) {
    .imr_abort("`threshold` must be between 0 and 1.")
  }
  selected <- vector("list", object$n_platform)
  names(selected) <- object$platform_names
  for (l in seq_len(object$n_platform)) {
    m <- .imr_mpip(object, l)
    if (nrow(m) == 0 || ncol(m) == 0) {
      selected[[l]] <- data.frame(feature = character(0), max_mpip = numeric(0),
                                  subgroup = character(0))
      next
    }
    maxp <- apply(m, 2, max)
    which_sg <- rownames(m)[apply(m, 2, which.max)]
    keep <- which(maxp > threshold)
    ord <- keep[order(maxp[keep], decreasing = TRUE)]
    selected[[l]] <- data.frame(
      feature = colnames(m)[ord],
      max_mpip = round(maxp[ord], 3),
      subgroup = which_sg[ord],
      row.names = NULL,
      stringsAsFactors = FALSE
    )
  }
  out <- list(
    call = object$call,
    type_outcome = object$type_outcome,
    method = object$method,
    threshold = threshold,
    sample_size = object$sample_size,
    model_bitstrings = object$model_bitstrings,
    platform_names = object$platform_names,
    selected = selected
  )
  class(out) <- "summary.imr"
  out
}

#' @rdname summary.imr
#' @param x A `"summary.imr"` object.
#' @export
print.summary.imr <- function(x, ...) {
  cat("Integrative Bayesian Multi-Platform Regression (IMR) -- summary\n")
  cat("--------------------------------------------------------------\n")
  cat(sprintf("Outcome type : %s   Method: %s\n", x$type_outcome, x$method))
  cat(sprintf("Selection threshold (mPIP) : %.2f\n\n", x$threshold))
  for (l in seq_along(x$selected)) {
    df <- x$selected[[l]]
    cat(sprintf("Platform '%s': %d selected feature(s)\n",
                x$platform_names[l], nrow(df)))
    if (nrow(df) > 0) {
      print(df, row.names = FALSE)
    }
    cat("\n")
  }
  invisible(x)
}


#' Plot Method for IMR Fits
#'
#' @description
#' Visualizes a fitted `"imr"` object.  Three plot types are available:
#' \describe{
#'   \item{`"selection"`}{Heatmap of the marginal posterior inclusion
#'     probabilities (mPIP), one panel per platform, with features on the
#'     horizontal axis, availability subgroups on the vertical axis and a small
#'     intensity legend showing that darker values are closer to 1.}
#'   \item{`"theta"`}{Heatmap of the posterior mean MRF interaction parameters
#'     between availability subgroups, one panel per platform.}
#'   \item{`"trace"`}{Trace plot of the log-posterior across MCMC iterations.}
#' }
#' See [plot_top_features()] and [plot_subgroup_sizes()] for two further ready
#' made displays.
#'
#' @param x A fitted object of class `"imr"`.
#' @param type Character; one of `"selection"` (default), `"theta"` or
#'   `"trace"`.
#' @param platform Optional integer vector selecting which platforms to display
#'   for the `"selection"` and `"theta"` plots; defaults to all platforms.
#' @param base_cex Overall text-size multiplier. The `cex_*` arguments default
#'   to values derived from this multiplier (default `1`).
#' @param cex_axis Axis-label size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param cex_lab Axis-title size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param cex_main Main-title size multiplier. If `NULL`, a plot-specific
#'   default derived from `base_cex` is used.
#' @param col Optional colours. For heatmaps this is the colour scale; for
#'   trace plots this is the line colour.
#' @param palette Optional heatmap palette name used when `col = NULL`.
#'   Selection plots default to `"grey"` to preserve the mPIP intensity scale;
#'   theta plots default to the muted grey-blue `"heatmap"` palette.
#' @param legend Logical; for `"selection"` plots, should the mPIP intensity
#'   legend be drawn (default `TRUE`)?
#' @param legend_width Relative width of the intensity-legend panel for
#'   `"selection"` plots (default `0.28`).
#' @param mar,mgp Optional graphical margin and axis-title placement vectors
#'   passed to [graphics::par()] for finer layout control.
#' @param ... Further graphical parameters passed to the underlying plotting
#'   functions.
#' @return `NULL`, invisibly; called for the side effect of producing a plot.
#' @seealso [imr()], [plot_top_features()], [plot_subgroup_sizes()]
#' @export
plot.imr <- function(x, type = c("selection", "theta", "trace"),
                     platform = NULL, base_cex = 1, cex_axis = NULL,
                     cex_lab = NULL, cex_main = NULL, col = NULL,
                     palette = NULL,
                     legend = TRUE, legend_width = 0.28,
                     mar = NULL, mgp = NULL, ...) {
  if (!inherits(x, "imr")) {
    .imr_abort("`x` must be an `imr` object returned by `imr()`.")
  }
  type <- match.arg(type)
  dots <- list(...)
  cex_defaults <- if (type == "selection") {
    list(axis = 1.05, lab = 1.3, main = 1.4,
         names = 1, legend = 1, values = 1)
  } else {
    list(axis = 0.95, lab = 1.1, main = 1.15,
         names = 1, legend = 1, values = 1)
  }
  sz <- .imr_plot_cex(
    dots, base_cex = base_cex, cex_axis = cex_axis,
    cex_lab = cex_lab, cex_main = cex_main,
    defaults = cex_defaults
  )
  cex_axis <- sz$axis
  cex_lab <- sz$lab
  cex_main <- sz$main
  dots <- sz$dots
  .imr_check_flag(legend, "legend")
  legend_width <- .imr_check_numeric_vector(
    legend_width, "legend_width", length = 1, positive = TRUE
  )
  if (!is.null(platform)) {
    if (!is.numeric(platform) || length(platform) == 0L ||
        any(!is.finite(platform)) || any(platform != as.integer(platform)) ||
        any(platform < 1L) || any(platform > x$n_platform)) {
      .imr_abort(sprintf(
        "`platform` must contain whole-number indices between 1 and %d.",
        x$n_platform
      ))
    }
    platform <- as.integer(platform)
  }

  op <- graphics::par(no.readonly = TRUE)
  on.exit({
    if (type == "selection") try(graphics::layout(1), silent = TRUE)
    try(graphics::par(op), silent = TRUE)
  }, add = TRUE)

  if (type == "trace") {
    lp <- x$log_posterior
    pp <- .imr_plot_par(mar, mgp, default_mar = c(4.8, 4.8, 3, 1))
    graphics::par(mar = pp$mar, mgp = pp$mgp)
    trace_col <- if (is.null(col)) .imr_plot_trace_colour() else col
    do.call(graphics::plot, c(list(
      x = seq_along(lp), y = lp, type = "l",
      xlab = "MCMC iteration", ylab = "Log-posterior",
      main = "Log-posterior trace", cex.axis = cex_axis,
      cex.lab = cex_lab, cex.main = cex_main, col = trace_col
    ), dots))
    graphics::abline(v = x$sample_mcmc[["burnin"]], lty = 2, col = "grey50")
    return(invisible(NULL))
  }

  plats <- if (is.null(platform)) seq_len(x$n_platform) else platform
  heat_palette <- if (is.null(palette)) {
    if (type == "selection") "grey" else "heatmap"
  } else {
    palette
  }
  heat_col <- if (is.null(col)) {
    .imr_plot_palette(64, palette = heat_palette)
  } else {
    .imr_plot_palette(length(col), col = col)
  }

  if (type == "selection") {
    layout_widths <- rep(1, length(plats))
    if (legend) layout_widths <- c(layout_widths, legend_width)
    graphics::layout(
      matrix(seq_along(layout_widths), nrow = 1L),
      widths = layout_widths
    )
  } else if (length(plats) > 1) {
    graphics::par(mfrow = c(1, length(plats)))
  }

  panel_par <- .imr_plot_par(
    mar, mgp,
    default_mar = if (type == "selection") {
      c(5.7, 4.8, 2.9, 0.5)
    } else {
      c(4.8, 4.8, 3, 1)
    },
    default_mgp = if (type == "selection") c(3.6, 0.9, 0) else c(2.7, 0.8, 0)
  )
  for (l in plats) {
    if (type == "selection") {
      m <- .imr_mpip(x, l)
      main <- sprintf("mPIP: %s", x$platform_names[l])
      xlab <- "Features"; ylab <- "Availability subgroups"
      rlab <- rownames(m)
    } else {
      m <- x$theta_mean[[l]]
      rlab <- x$model_bitstrings[x$platform_models[[l]]]
      if (!is.null(rlab) && length(rlab) == nrow(m)) {
        rownames(m) <- colnames(m) <- rlab
      }
      main <- sprintf("Theta: %s", x$platform_names[l])
      xlab <- "Availability subgroups"; ylab <- "Availability subgroups"
    }
    graphics::par(mar = panel_par$mar, mgp = panel_par$mgp)
    if (nrow(m) == 0 || ncol(m) == 0) {
      graphics::plot.new()
      graphics::title(main = paste(main, "(empty)"), cex.main = cex_main)
      next
    }
    zlim <- if (type == "selection") c(0, 1) else range(m, na.rm = TRUE)
    do.call(graphics::image, c(list(
      x = seq_len(ncol(m)), y = seq_len(nrow(m)), z = t(m),
      col = heat_col, zlim = zlim, axes = FALSE,
      xlab = xlab, ylab = ylab, main = main,
      cex.lab = cex_lab, cex.main = cex_main
    ), dots))
    if (!is.null(colnames(m)) && ncol(m) <= 40) {
      graphics::axis(1, at = seq_len(ncol(m)), labels = colnames(m),
                     las = 2, cex.axis = cex_axis)
    } else {
      graphics::axis(1, cex.axis = cex_axis)
    }
    graphics::axis(2, at = seq_len(nrow(m)), labels = rlab, las = 2,
                   cex.axis = cex_axis)
    graphics::box()
  }
  if (type == "selection" && legend) {
    legend_par <- .imr_plot_par(
      NULL, mgp, default_mar = c(5.7, 0.1, 2.9, 3.0),
      default_mgp = c(3.6, 0.9, 0)
    )
    graphics::par(mar = legend_par$mar, mgp = legend_par$mgp)
    graphics::plot.new()
    graphics::plot.window(xlim = c(0, 1), ylim = c(0, 1))
    yb <- seq(0, 1, length.out = length(heat_col) + 1L)
    graphics::rect(0.18, yb[-length(yb)], 0.52, yb[-1L],
                   col = heat_col, border = NA)
    graphics::axis(4, at = c(0, 0.25, 0.5, 0.75, 1),
                   labels = c("0", "0.25", "0.5", "0.75", "1"),
                   las = 1, cex.axis = cex_axis, tck = -0.18)
    graphics::mtext("mPIP", side = 3, line = 0.1, at = 0.35,
                    cex = 0.75 * cex_main)
    graphics::box(bty = "n")
  }
  invisible(NULL)
}

#' Cross-Validated Predictive Accuracy of an IMR Fit
#'
#' @description
#' `cv_imr()` evaluates predictive accuracy of a model fitted with [imr()] using
#' repeated \eqn{K}-fold splits and Bayesian model averaging over the fitted
#' MCMC samples.  The sampler is not re-run inside each training fold; fit a
#' separate object with `method = "BMS"` if a no-borrowing comparison is needed.
#' The accuracy measure depends on the outcome type: the concordance index
#' (C-index) for right-censored outcomes, the area under the ROC curve (AUC) for
#' binary outcomes, and the mean squared error (MSE) for continuous outcomes.
#'
#' @param object A fitted object of class `"imr"` returned by [imr()].
#' @param k Integer number of cross-validation folds per round (default `5`).
#'   Must be at least `2`, and each availability subgroup must contain at least
#'   `k` subjects.
#' @param rounds Integer number of independent cross-validation rounds to
#'   average over (default `2`).  Must be positive.
#' @param method Optional compatibility argument.  If supplied, it must match
#'   the method stored in `object`; `cv_imr()` cannot turn an IMR fit into a BMS
#'   fit or vice versa.
#' @param max_models Integer maximum number of selection models used for
#'   Bayesian model averaging (default `100`).  Must be positive.
#' @param verbose Logical; if `TRUE`, print the C routine's cross-validation
#'   diagnostics.  Defaults to `FALSE`.
#'
#' @return A list with two numeric matrices, each of dimension
#'   `rounds` x `(n_subgroups + 1)`, whose last column corresponds to all
#'   subjects pooled and whose remaining columns are named by the availability
#'   subgroup bitstrings:
#'   \item{total_cindex}{Accuracy computed on the pooled cross-validated
#'     predictions within each availability subgroup (and overall).}
#'   \item{subset_cindex}{Accuracy computed fold-by-fold within each
#'     availability subgroup.}
#'   The accuracy metric (`"C-index"`, `"AUC"` or `"MSE"`) is recorded in the
#'   `"metric"` attribute of the returned list.
#'
#' @seealso [imr()], [predict.imr()]
#'
#' @examples
#' \donttest{
#' data("simIMR", package = "IntegMultiReg")
#' fit <- imr(
#'   platform_data_list = simIMR$platforms, outcome = simIMR$outcome,
#'   cov = simIMR$covariates, type_outcome = "binary",
#'   nu = c(-4, -3, -4), sample_mcmc = c(200, 100), ssize = 5, seed = 1
#' )
#' cv <- cv_imr(fit, k = 5, rounds = 2)
#' cv$total_cindex
#' }
#' @export
cv_imr <- function(object, k = 5, rounds = 2,
                   method = NULL,
                   max_models = 100, verbose = FALSE) {
  if (!inherits(object, "imr")) {
    .imr_abort("`object` must be an `imr` object returned by `imr()`.")
  }
  .imr_check_flag(verbose, "verbose")
  k <- .imr_check_integer_scalar(k, "k", min = 2)
  rounds <- .imr_check_integer_scalar(rounds, "rounds", min = 1)
  max_models <- .imr_check_integer_scalar(max_models, "max_models", min = 1)
  object_method <- object$method
  if (is.null(object_method)) object_method <- "IMR"
  if (is.null(method)) {
    method <- object_method
  } else {
    method <- match.arg(method, c("IMR", "BMS"))
    if (!identical(method, object_method)) {
      .imr_abort(
        "`method` must match the fitted object; fit a separate `imr(..., method = \"BMS\")` object for BMS validation."
      )
    }
  }
  if (any(object$sample_size < k)) {
    .imr_abort(
      "`k` must not exceed the sample size of any modelled availability subgroup."
    )
  }
  n_platform <- object$data1[[1]]
  method_c <- as.character(method)
  type_out <- object$data1[[8]]
  metric <- c("C-index", "AUC", "MSE")[type_out]

  results <- .quietly(verbose, .Call("mainFunctionPrediction",
    h0_c = object$list_hyperpara[1],
    hh_c = object$list_hyperpara[2],
    alpha_c = object$list_hyperpara[3],
    psi_c = object$list_hyperpara[4],
    alpha0_c = object$list_hyperpara[5],
    beta0_c = object$list_hyperpara[6],
    seed_c = object$list_hyperpara[7],
    nu_c = object$list_hyperpara[8:(7 + n_platform)],
    y_latent = object$estimate_latent_y,
    gam_sample_c = object$gam_sample,
    theta_c = object$theta_mean,
    method_c = method_c,
    n_platform_c = as.integer(n_platform),
    platform_models_c = object$data1[[2]],
    model_platforms_c = object$data1[[3]],
    n_models = as.integer(object$data1[[4]]),
    sample_size = as.integer(object$data1[[5]]),
    n_features = as.integer(object$data1[[6]]),
    n_cov = as.integer(object$data1[[7]]),
    x_filtered = object$data2[[1]],
    y_list = object$data2[[2]],
    cov_list = object$data2[[3]],
    type_outcome = as.integer(object$data1[[8]]),
    sample = as.integer(object$data1[[9]]),
    kcv = as.integer(k),
    round10cv = as.integer(rounds),
    max_models_pred = as.integer(max_models)
  ))

  ## Label the columns: one per availability subgroup plus a pooled "all" column.
  col_labels <- c(object$model_bitstrings, "all")
  if (!is.null(results$total_cindex) &&
      ncol(results$total_cindex) == length(col_labels)) {
    colnames(results$total_cindex) <- col_labels
    colnames(results$subset_cindex) <- col_labels
  }
  attr(results, "metric") <- metric
  return(results)
}

#' Simulated Multi-Platform Example Data
#'
#' A small simulated data set illustrating the integrative multi-regression
#' (IMR) setting: three molecular platforms measured on overlapping but
#' partially missing sets of subjects, clinical covariates available for
#' everyone, and three outcome types (continuous, binary and right-censored)
#' driven by the same latent signal.
#'
#' The platforms are observed on the following subjects, which induces four
#' non-empty availability subgroups (bitstrings over genomic/proteomic/
#' metabolomic): `011` (120 subjects), `111` (60), `101` (60) and `100` (60).
#'
#' @format A named list with the components:
#' \describe{
#'   \item{platforms}{A list of three data frames, `genomic` (20 features,
#'     subjects 1--240), `proteomic` (10 features, subjects 1--180) and
#'     `metabolomic` (8 features, subjects 121--300).  Each has an `id` column
#'     followed by the feature columns.}
#'   \item{covariates}{A data frame of three clinical covariates (`age`, `sex`,
#'     `stage`) with an `id` column, available for all 300 subjects.}
#'   \item{outcome}{The binary outcome data frame (alias of
#'     `outcome.binary`), used as the canonical demonstration outcome.}
#'   \item{outcome.binary}{Data frame with `id` and a 0/1 response `y`.}
#'   \item{outcome.continuous}{Data frame with `id` and a numeric response `y`.}
#'   \item{outcome.survival}{Data frame with `id`, observed `time` and event
#'     indicator `status` (1 = event, 0 = censored).}
#'   \item{truth}{A list giving, per platform, the column indices of the
#'     features with non-zero effects used to generate the outcomes.}
#' }
#'
#' @details The generating script is provided in `data-raw/make_simIMR.R`.
#'
#' @examples
#' data("simIMR", package = "IntegMultiReg")
#' sapply(simIMR$platforms, dim)
#' str(simIMR$truth)
#' @keywords datasets
"simIMR"


#' Reduced TCGA-KIRC Multi-Platform Survival Example
#'
#' A reduced real-data example derived from public UCSC Xena TCGA kidney renal
#' clear cell carcinoma (KIRC) data, not from controlled-access TCGA/GDC files.
#' The data set mirrors the multi-platform structure of Chekouo, Stingo, Doecke
#' and Do (2017): mRNA expression, miRNA expression and DNA methylation are
#' observed on overlapping but not identical sets of patients, with clinical
#' covariates and right-censored survival outcome.
#'
#' The full TCGA source files are not distributed with the package.  Instead,
#' this object stores a small screened feature panel suitable for examples and
#' vignette analyses.  Patient identifiers are package-internal labels
#' (`KIRC001`, `KIRC002`, ...) rather than TCGA barcodes; no barcode mapping is
#' distributed with the package.  Users should not attempt participant
#' re-identification or linkage to external resources.  The generating script is
#' provided in the source repository under `data-raw/make_kircIMR.R`.
#'
#' @format A named list with the components:
#' \describe{
#'   \item{platforms}{A list of three data frames, `mrna`, `mirna` and
#'     `methylation`.  Each has an `id` column containing package-internal
#'     patient labels followed by screened molecular features.}
#'   \item{covariates}{Clinical covariates with an `id` column: age at initial
#'     diagnosis, female sex indicator, pathologic stage and histologic grade.}
#'   \item{outcome}{Alias of `outcome.survival`, for convenience.}
#'   \item{outcome.survival}{Data frame with `id`, observed survival/follow-up
#'     time in days and event indicator `status` (1 = deceased, 0 = censored).}
#'   \item{feature_screening}{Per-platform feature-ranking information and a
#'     short description of the screening procedure used to create the reduced
#'     panel.}
#'   \item{platform_availability}{Patient-level logical matrix indicating which
#'     of the three platforms are available.}
#'   \item{subgroup_sizes}{Counts of the non-empty availability subgroups
#'     induced by the reduced data.}
#'   \item{model_subgroup_sizes}{The availability subgroups that would be
#'     modelled under the default real-data illustration threshold
#'     `ssize = 30`.}
#'   \item{paper_alignment}{Metadata describing how the reduced example maps to
#'     the Biometrics KIRC case study.}
#'   \item{source}{Source URLs and reference metadata.}
#' }
#'
#' @details The source data are public UCSC Xena TCGA-KIRC sampleMap files:
#' mRNA expression (`HiSeqV2`), miRNA expression (`miRNA_HiSeq_gene`), DNA
#' methylation (`HumanMethylation450`) and the KIRC clinical matrix.  Molecular
#' features are screened with univariable Cox models after basic filtering, and
#' missing molecular values in the retained panel are median-imputed within
#' feature.  The full Biometrics analysis used 776 mRNA, 91 miRNA and 729 DNA
#' methylation features; this package example retains 50, 30 and 50 features,
#' respectively, to keep examples lightweight.  The package does not distribute
#' controlled-access TCGA/GDC data or the full Biometrics/Wiley supplementary
#' data files.
#'
#' @references Chekouo T, Stingo FC, Doecke JD, Do K-A (2017). "A Bayesian
#' Integrative Approach for Multi-Platform Genomic Data: A Kidney Cancer Case
#' Study." \emph{Biometrics}, \strong{73}(2), 615--624.
#' \doi{10.1111/biom.12587}
#'
#' The Cancer Genome Atlas Research Network; National Cancer Institute Genomic
#' Data Commons; UCSC Xena.
#'
#' @examples
#' data("kircIMR", package = "IntegMultiReg")
#' sapply(kircIMR$platforms, dim)
#' kircIMR$subgroup_sizes
#' kircIMR$model_subgroup_sizes
#' @keywords datasets
"kircIMR"

#' Fit the Integrative Bayesian Multi-Platform Regression Model
#'
#' @description
#' `imr()` fits the integrative multi-regression (IMR) model of Chekouo et al.
#' (2017) by Markov chain Monte Carlo (MCMC).
#' It identifies biomarkers associated with a time-to-event, binary or
#' continuous outcome while borrowing information across all subjects,
#' regardless of which platforms each subject has measured.  Subjects are
#' partitioned into availability subgroups defined by their pattern of platform
#' availability; one regression is built per subgroup and information is shared
#' across subgroups through a Markov random field (MRF) prior on the
#' variable-selection indicators together with non-local priors on the
#' regression coefficients.  The sampler is implemented in C for efficiency.
#'
#' The arguments are grouped by the orthogonal aspect of the analysis that each
#' one controls: the *data* (`platform_data_list`, `outcome`, `cov`), the
#' *likelihood* (`type_outcome`), the *model* (`method`), subgroup *filtering*
#' (`ssize`), the *priors* (`nu`, `hh`, `h0`, `sig_alpha_psi`, `thet_alph_bet`),
#' the *computation* (`sample_mcmc`, `seed`) and the *output* (`verbose`).
#'
#' @param platform_data_list A list of data frames, one per platform.  Each
#'   data frame must contain an `id` column (the subject identifier, taken to be
#'   the first column); the remaining columns are finite numeric features
#'   measured on that platform.  Subject identifiers must be unique within each
#'   data frame.  The `id` column links subjects across platforms and to the
#'   outcome and covariate data.
#' @param outcome A data frame containing an `id` column and the response.  For
#'   `type_outcome = "right.censored"` it must also contain the event/censoring
#'   time and a censoring indicator (three columns in total).  For `"binary"`
#'   the response must be coded 0/1 and for `"continuous"` it is a numeric
#'   response (two columns in total).  The `id` column must be first and unique.
#' @param cov An optional data frame of clinical covariates including an `id`
#'   column followed by finite numeric covariates.  Covariates are always
#'   included in every regression (they are not subject to selection).  Defaults
#'   to `NULL` (no covariates).
#' @param type_outcome Character string specifying the outcome type, one of
#'   `"right.censored"` (default), `"binary"` or `"continuous"`.
#' @param method Character string specifying the method, `"IMR"` (default) for
#'   the integrative model that shares information across subgroups via the MRF
#'   prior, or `"BMS"` for the non-integrative Bayesian multi-step variant that
#'   fits each subgroup independently (MRF interaction parameters set to zero).
#' @param ssize Minimum availability subgroup size for a subgroup to be
#'   modelled. Subgroups with at most `ssize` subjects are dropped. Default is
#'   `30`.
#' @param nu A numeric vector of prior log-odds of inclusion, one value per
#'   platform, controlling the prior sparsity of the selected features.
#'   Defaults to `rep(-3, length(platform_data_list))`.
#' @param hh Scale of the non-local (product moment) prior on the slab
#'   regression effects (default `0.087`).
#' @param h0 Prior variance of the intercept (default `10000`).
#' @param sig_alpha_psi Length-2 numeric vector with the shape and rate of the
#'   inverse-gamma prior on the response error variance (default
#'   `c(0.001, 0.001)`).  Ignored when `type_outcome = "binary"`: a probit model
#'   has residual variance fixed at 1 for identifiability, so the error variance
#'   is pinned to 1 rather than estimated.
#' @param thet_alph_bet Length-2 numeric vector with the shape and rate of the
#'   gamma prior on the MRF interaction parameters theta, which borrow
#'   information across subgroups (default `c(40, 10)`).
#' @param sample_mcmc An integer vector `c(n_retained, n_burnin)` giving the
#'   number of post-burn-in draws to retain and the number of burn-in
#'   iterations to discard.  The sampler runs `n_retained + n_burnin`
#'   iterations in total, so the returned `log_posterior` has
#'   `sum(sample_mcmc)` entries.  Default is `c(2000, 1000)`.
#' @param seed Optional integer seed for the sampler.  If `NULL` (the default) a
#'   seed is drawn from the current R RNG state, so a run is reproducible
#'   whenever [set.seed()] is called beforehand or an explicit `seed` is passed.
#' @param verbose Logical; if `TRUE`, print the sampler's progress and
#'   diagnostics to the console.  Defaults to `FALSE` (quiet).
#'
#' @details
#' All feature and covariate data are standardized internally (mean 0, standard
#' deviation 1); the centring and scaling factors are stored in the returned
#' object so that [predict.imr()] can apply the same transformation to new
#' subjects.  For right-censored outcomes the latent log-survival times of
#' censored subjects are imputed within the sampler; for binary outcomes a
#' probit data-augmentation latent variable is sampled.
#'
#' @return An object of class `"imr"`: a list with components
#' \item{gam_mean}{List (one matrix per platform) of posterior mean
#'   variable-selection probabilities; rows index the subgroups containing the
#'   platform and columns index the platform features.}
#' \item{theta_mean}{List (one matrix per platform) of posterior mean MRF
#'   interaction parameters between subgroups.}
#' \item{estimate_latent_y}{List (one vector per subgroup) of posterior mean
#'   latent responses, useful for censored or binary data.}
#' \item{log_posterior}{Numeric vector of log-posterior values across all MCMC
#'   iterations (burn-in included).}
#' \item{gam_sample}{Post-burn-in MCMC samples of the selection indicators.}
#' \item{theta_sample}{Post-burn-in MCMC samples of the MRF interaction
#'   parameters (absent when `method = "BMS"`).}
#' \item{list_hyperpara, data1, data2}{Hyper-parameters and pre-processed data
#'   retained for prediction and cross-validation.}
#' \item{call, type_outcome, n_platform, platform_names, feature_names,
#'   covariate_names, model_bitstrings, sample_size, model_platforms,
#'   platform_models, sample_mcmc, nu, method}{Run metadata used by the print,
#'   summary, plot and predict methods.}
#'
#' @references
#' Chekouo T, Stingo FC, Doecke JD, Do K-A (2017). "A Bayesian Integrative
#' Approach for Multi-Platform Genomic Data: A Kidney Cancer Case Study."
#' \emph{Biometrics}, \strong{73}(2), 615--624. \doi{10.1111/biom.12587}
#'
#' @seealso [predict.imr()], [cv_imr()], [summary.imr()], [plot.imr()]
#'
#' @examples
#' \donttest{
#' data("simIMR", package = "IntegMultiReg")
#' fit <- imr(
#'   platform_data_list = simIMR$platforms,
#'   outcome = simIMR$outcome,
#'   cov = simIMR$covariates,
#'   type_outcome = "binary",
#'   nu = c(-4, -3, -4),
#'   sample_mcmc = c(200, 100),
#'   ssize = 5,
#'   seed = 1
#' )
#' fit
#' }
#' @export
imr <- function(platform_data_list,
                                           outcome,
                                           cov = NULL,
                                           type_outcome = c("right.censored", "binary", "continuous"),
                                           method = c("IMR", "BMS"),
                                           ssize = 30,
                                           nu = rep(-3, length(platform_data_list)),
                                           hh = 0.087,
                                           h0 = 10000,
                                           sig_alpha_psi = c(0.001, 0.001),
                                           thet_alph_bet = c(40, 10),
                                           sample_mcmc = c(2000, 1000),
                                           seed = NULL,
                                           verbose = FALSE) {
  cl <- match.call()
  type_outcome <- match.arg(type_outcome)
  method <- match.arg(method)

  .imr_check_flag(verbose, "verbose")
  if (!is.list(platform_data_list) || length(platform_data_list) == 0L) {
    .imr_abort("`platform_data_list` must be a non-empty list of data frames.")
  }
  n_platform <- length(platform_data_list)
  for (i in seq_along(platform_data_list)) {
    arg <- sprintf("platform_data_list[[%d]]", i)
    .imr_check_id_frame(platform_data_list[[i]], arg)
    .imr_check_numeric_columns(platform_data_list[[i]], arg)
  }

  .imr_check_id_frame(outcome, "outcome")
  if (type_outcome %in% c("binary", "continuous")) {
    if (ncol(outcome) != 2L) {
      .imr_abort(
        "`outcome` must have exactly two columns: `id` and the response."
      )
    }
    .imr_check_numeric_columns(outcome, "outcome", names(outcome)[2])
    if (type_outcome == "binary" && !all(outcome[[2]] %in% c(0, 1))) {
      .imr_abort("For `type_outcome = \"binary\"`, the response must be coded 0/1.")
    }
  } else if (type_outcome == "right.censored") {
    if (ncol(outcome) != 3L) {
      .imr_abort(
        paste0("`outcome` must have exactly three columns for right-censored ",
               "data: `id`, time and status.")
      )
    }
    .imr_check_numeric_columns(outcome, "outcome", names(outcome)[2:3])
    if (any(outcome[[2]] <= 0)) {
      .imr_abort("Right-censored event times in `outcome` must be positive.")
    }
    if (!all(outcome[[3]] %in% c(0, 1))) {
      .imr_abort("Right-censored status values in `outcome` must be coded 0/1.")
    }
  }

  if (!is.null(cov)) {
    .imr_check_id_frame(cov, "cov")
    .imr_check_numeric_columns(cov, "cov")
  }

  ssize <- .imr_check_integer_scalar(ssize, "ssize", min = 0)
  nu <- .imr_check_numeric_vector(nu, "nu", length = n_platform)
  h0 <- .imr_check_numeric_vector(h0, "h0", length = 1, positive = TRUE)
  hh <- .imr_check_numeric_vector(hh, "hh", length = 1, positive = TRUE)
  sig_alpha_psi <- .imr_check_numeric_vector(
    sig_alpha_psi, "sig_alpha_psi", length = 2, positive = TRUE
  )
  thet_alph_bet <- .imr_check_numeric_vector(
    thet_alph_bet, "thet_alph_bet", length = 2, positive = TRUE
  )
  sample_mcmc <- .imr_check_integer_vector(
    sample_mcmc, "sample_mcmc", length = 2, min = 0
  )
  if (sample_mcmc[1] <= 0L) {
    .imr_abort("`sample_mcmc[1]` (retained draws) must be positive.")
  }
  if (!is.null(seed)) {
    seed <- .imr_check_integer_scalar(seed, "seed", min = 0)
  }

  ## The sampler draws on both R's RNG and a GSL RNG seeded by `seed`.  With an
  ## explicit seed we call set.seed() so the run is fully reproducible; when
  ## `seed` is NULL we instead draw one from the ambient R RNG, so results follow
  ## the user's own set.seed() like other R modelling functions.
  if (is.null(seed)) {
    seed <- sample.int(.Machine$integer.max, 1L)
  } else {
    set.seed(seed)
  }
  type_out <- 1
  if (type_outcome == "binary") {
    type_out <- 2
  } else if (type_outcome == "continuous") {
    type_out <- 3
  }

  ## Record human-readable platform and feature names (the first column of each
  ## platform is the 'id' and is dropped before modelling).
  platform_names <- names(platform_data_list)
  if (is.null(platform_names) || any(platform_names == "")) {
    platform_names <- paste0("platform", seq_len(n_platform))
  }
  feature_names <- lapply(platform_data_list, function(df) colnames(df)[-1])

  dat <- subgroup_data(outcome, cov, platform_data_list)
  n_sample <- sample_mcmc[1]
  n_burnin <- sample_mcmc[2]
  # Prepare scalar parameters.
  h0_c <- as.numeric(h0)
  hh_c <- as.numeric(hh)
  alpha_c <- as.numeric(sig_alpha_psi[1])
  psi_c <- as.numeric(sig_alpha_psi[2])
  if (type_outcome == "binary") {
    # A probit outcome fixes the residual variance at 1 for identifiability
    # (the latent utility is z = eta + e with e ~ N(0, 1)).  The marginal
    # likelihood otherwise integrates sigma^2 out under this inverse-gamma
    # prior, which leaves the latent scale only weakly identified and makes the
    # binary chain drift and mix poorly.  Concentrating the prior at 1 pins
    # sigma^2 = 1; any large shape/rate gives an effectively fixed unit variance.
    probit_unit_variance <- 1e5
    alpha_c <- psi_c <- probit_unit_variance
  }
  alpha0_c <- as.numeric(thet_alph_bet[1])
  beta0_c <- as.numeric(thet_alph_bet[2])
  seed_c <- as.numeric(seed)

  storage.mode(h0_c) <- "double"
  storage.mode(hh_c) <- "double"
  storage.mode(alpha_c) <- "double"
  storage.mode(psi_c) <- "double"
  storage.mode(alpha0_c) <- "double"
  storage.mode(beta0_c) <- "double"
  storage.mode(seed_c) <- "double"

  method_c <- as.character(method)

  #################################################################
  # Process the platform data and extract subgroup id vectors.
  #################################################################
  dat_orig <- dat # Preserve the original data (with "id" columns intact)
  # For each subgroup, extract the id vector from the first nonempty platform.
  dat_ids <- lapply(dat_orig[[3]], function(subgroup) {
    nonempty <- which(sapply(subgroup, function(platform_data) {
      if (is.data.frame(platform_data) && "id" %in% colnames(platform_data)) {
        return(nrow(platform_data))
      } else {
        return(0)
      }
    }) > 0)
    if (length(nonempty) > 0) {
      return(subgroup[[nonempty[1]]]$id)
    } else {
      return(character(0))
    }
  })

  # Now convert the platform data into a list of numeric matrices by dropping
  # the "id" column.
  dat[[3]] <- lapply(dat[[3]], function(subgroup) {
    lapply(subgroup, function(platform_data) {
      if (is.data.frame(platform_data) && "id" %in% colnames(platform_data)) {
        mat <- as.matrix(platform_data[, -1, drop = FALSE])
      } else if (!is.null(colnames(platform_data)) &&
        colnames(platform_data)[1] == "id") {
        mat <- as.matrix(platform_data[, -1, drop = FALSE])
      } else {
        mat <- as.matrix(platform_data)
      }
      storage.mode(mat) <- "double"
      return(mat)
    })
  })

  n_features <- as.integer(vapply(
    platform_data_list, function(x) ncol(x) - 1L, integer(1)
  ))
  # Compute representative row counts for each subgroup.
  subgroup_rows <- sapply(dat[[3]], function(subgroup) {
    nonempty <- which(sapply(subgroup, function(mat) nrow(mat)) > 0)
    if (length(nonempty) > 0) {
      nrow(subgroup[[nonempty[1]]])
    } else {
      0
    }
  })

  # Keep only subgroups with more than 'ssize' rows.
  model_index <- as.integer(which(subgroup_rows > ssize))
  if (length(model_index) == 0) {
    .imr_abort(
      "No availability subgroup has more than `ssize` subjects; lower `ssize` or check the data."
    )
  }
  sample_size <- as.integer(subgroup_rows[model_index])

  # Filter the platform data to include only subgroups meeting the threshold.
  dat_filtered <- lapply(dat, function(x) x[model_index])

  # Also filter the original id vectors accordingly.
  subgroup_ids_filtered <- dat_ids[model_index]

  #############################################################
  # Process the response and covariates using the filtered subgroup ids.
  #############################################################
  ### Match ids and drop the id column from the outcome and covariate frames.
  dat_filtered[[1]] <- lapply(
    seq_along(dat_filtered[[1]]),
    function(i) {
      .imr_match_rows(
        dat_filtered[[1]][[i]], subgroup_ids_filtered[[i]], "outcome"
      )[, -1, drop = FALSE]
    }
  )
  if (!is.null(cov)) {
    dat_filtered[[2]] <- lapply(
      seq_along(dat_filtered[[2]]),
      function(i) {
        .imr_match_rows(
          dat_filtered[[2]][[i]], subgroup_ids_filtered[[i]], "cov"
        )[, -1, drop = FALSE]
      }
    )
  }

  dat_normalized <- dat_filtered

  mean_train <- mean_nested_list(dat_filtered[[3]])
  sd_train <- sd_nested_list(dat_filtered[[3]])
  if (!is.null(cov)) {
    mean_cov_train <- lapply(dat_filtered[[2]], mean_matrix)
    sd_cov_train <- lapply(dat_filtered[[2]], sd_matrix)
  } else {
    mean_cov_train <- NULL
    sd_cov_train <- NULL
  }

  dat_normalized[[3]] <- normalize_nested_list(dat_filtered[[3]])
  ### Normalized covariates
  if (!is.null(cov)) {
    dat_normalized[[2]] <- lapply(dat_filtered[[2]], normalize_matrix)
  }
  ## outcome: force double storage (the C sampler reads it with REAL())
  dat_normalized[[1]] <- lapply(dat_filtered[[1]], function(x) {
    m <- as.matrix(x)
    storage.mode(m) <- "double"
    m
  })

  if (verbose) {
    cat("Sample sizes for modelled availability subgroups:", sample_size, "\n")
  }

  ### For each model, list the corresponding platforms involved in the model
  n_models <- length(model_index)

  model_platforms_c <- sapply(1:n_models, function(x) {
    as.integer((seq_along(dat_normalized[[3]][[x]]) - 1)[unlist(lapply(
      seq_along(dat_normalized[[3]][[x]]),
      function(i) nrow(dat_normalized[[3]][[x]][[i]])
    )) > 0])
  }, simplify = FALSE)

  ### For each platform, obtain the model indices where that platform is involved
  platform_models_c <- sapply(1:n_platform, function(x) {
    as.integer((seq(1, n_models) - 1)[unlist(lapply(
      model_platforms_c,
      function(y) (x - 1) %in% y
    ))])
  }, simplify = FALSE)

  n_platform_c <- n_platform
  x_filtered <- dat_normalized[[3]]
  y_list <- dat_normalized[[1]]

  if (!is.null(cov)) {
    n_cov <- ncol(dat_normalized[[2]][[1]])
  } else {
    n_cov <- 0
  }
  if (n_cov == 0) {
    cov_list <- lapply(y_list, function(x) {
      matrix(numeric(0), nrow = nrow(x), ncol = 0)
    })
  } else {
    cov_list <- dat_normalized[[2]]
  }

  nu_c <- as.double(nu)

  ###########################################
  # Call the compiled MCMC sampler.
  ###########################################
  results <- .quietly(verbose, .Call("mainFunction", h0_c, hh_c, alpha_c, psi_c,
    alpha0_c, beta0_c, seed_c, nu_c, method_c,
    n_platform_c = as.integer(n_platform_c),
    platform_models_c = platform_models_c, model_platforms_c = model_platforms_c,
    n_models = as.integer(n_models),
    sample_size = as.integer(sample_size),
    n_features = as.integer(n_features),
    n_cov = as.integer(n_cov),
    x_filtered = x_filtered, y_list = y_list,
    type_outcome = as.integer(type_out),
    cov_list = cov_list,
    sample = as.integer(n_sample),
    burnin = as.integer(n_burnin)
  ))

  ## Guard against tiny floating-point drift in the running averages so that
  ## the reported inclusion probabilities are exactly within [0, 1].
  results$gam_mean <- lapply(results$gam_mean, function(m) {
    m[m > 1] <- 1
    m[m < 0] <- 0
    m
  })

  results$list_hyperpara <- c(
    h0_c, hh_c, alpha_c, psi_c, alpha0_c, beta0_c, seed_c, nu_c
  )
  # total number of hyper-parameters = 7 + n_platform

  results$data1 <- list(
    n_platform_c, platform_models_c, model_platforms_c,
    n_models, sample_size, n_features, n_cov, type_out,
    n_sample
  )
  results$data2 <- list(
    xx = x_filtered, yy = y_list, cc = cov_list,
    mean_train, sd_train, mean_cov_train, sd_cov_train
  )

  ## Run metadata for the print / summary / plot / predict methods.
  model_bitstrings <- names(x_filtered)
  results$call <- cl
  results$type_outcome <- type_outcome
  results$method <- method
  results$n_platform <- n_platform
  results$platform_names <- platform_names
  results$feature_names <- feature_names
  results$covariate_names <- if (!is.null(cov)) colnames(cov)[-1] else character(0)
  results$model_bitstrings <- model_bitstrings
  results$sample_size <- stats::setNames(sample_size, model_bitstrings)
  results$model_platforms <- lapply(model_platforms_c, function(x) x + 1L)
  results$platform_models <- lapply(platform_models_c, function(x) x + 1L)
  results$nu <- nu
  results$ssize <- ssize
  results$sample_mcmc <- c(total = n_sample, burnin = n_burnin)

  class(results) <- "imr"
  return(results)
}


#' @keywords internal
#' @noRd
subgroup_data <- function(outcome, cov = NULL, platform_data_list) {
  # Collect all the input data frames into a list
  nplat <- length(platform_data_list)
  ### Intersection of outcome and covariate ids with the union of platform ids
  ids_out_cov <- outcome$id
  if (!is.null(cov)) {
    ids_out_cov <- intersect(outcome$id, cov$id)
  }

  platforms <- lapply(platform_data_list, function(x) {
    x[x$id %in% ids_out_cov, , drop = FALSE]
  })

  id_outcome <- unique(unlist(lapply(platforms, function(x) x$id)))
  if (length(id_outcome) == 0L) {
    .imr_abort(
      "No subjects have both outcome/covariate data and at least one platform."
    )
  }
  outcome1 <- outcome[outcome$id %in% id_outcome, , drop = FALSE]
  cov1 <- if (!is.null(cov)) {
    cov[cov$id %in% id_outcome, , drop = FALSE]
  } else {
    NULL
  }

  # Check that each platform has at least one column named "id"
  if (!all(sapply(platforms, function(df) "id" %in% colnames(df)))) {
    .imr_abort("Each input data frame must have an `id` column.")
  }

  # Create the union of all ids across platforms
  all_ids <- unique(unlist(lapply(platforms, function(df) df$id)))

  # Build a presence matrix (rows = subjects, columns = platforms).
  presence <- do.call(cbind, lapply(platforms, function(df) all_ids %in% df$id))

  # Create a binary string for each subject.
  # Convention: the rightmost digit is presence in the first platform, etc.
  bitstrings <- apply(presence, 1, function(x) paste(as.integer(rev(x)), collapse = ""))

  # Identify the unique binary patterns (subgroups) sorted in ascending order.
  unique_patterns <- sort(unique(bitstrings))

  x1 <- list()
  sample_ids <- list()

  for (pat in unique_patterns) {
    subgroup_ids <- all_ids[bitstrings == pat]
    subgroup_list <- vector("list", nplat)
    names(subgroup_list) <- paste0("platform", seq_len(nplat))
    for (i in seq_len(nplat)) {
      # For the i-th platform, the corresponding bit is at position nplat - i + 1.
      bit <- substr(pat, nplat - i + 1, nplat - i + 1)
      if (bit == "1") {
        subgroup_list[[i]] <- .imr_match_rows(
          platforms[[i]], subgroup_ids, sprintf("platform_data_list[[%d]]", i)
        )
      } else {
        subgroup_list[[i]] <- platforms[[i]][FALSE, , drop = FALSE]
      }
    }
    sample_ids[[pat]] <- subgroup_ids
    x1[[pat]] <- subgroup_list
  }
  ## Align the outcome and covariate rows to each subgroup's canonical subject
  ## order (`sample_ids`, taken from the platform data) with match(), rather than
  ## relying on the inputs sharing the same row order.  The compiled sampler
  ## pairs outcome / covariate / platform rows by position, so this keeps them
  ## correctly aligned even when the outcome or covariate frames are supplied in
  ## a different order (for id-sorted inputs it is a no-op).
  outcome2 <- lapply(sample_ids, function(x) .imr_match_rows(outcome1, x, "outcome"))
  cov2 <- if (!is.null(cov1)) {
    lapply(sample_ids, function(x) .imr_match_rows(cov1, x, "cov"))
  } else {
    lapply(sample_ids, function(x) data.frame(id = x))
  }

  return(list(outcome = outcome2, covariate = cov2, platform_data = x1))
}

#' @keywords internal
#' @noRd
normalize_matrix <- function(mat) {
  if (nrow(mat) == 0 || ncol(mat) == 0) {
    return(matrix(numeric(0), nrow = nrow(mat), ncol = ncol(mat)))
  }
  nm <- apply(mat, 2, function(col) {
    m <- mean(col, na.rm = TRUE)
    s <- sd(col, na.rm = TRUE)
    if (is.na(s) || s == 0) {
      rep(0, length(col))
    } else {
      (col - m) / s
    }
  })
  if (is.null(dim(nm))) {
    nm <- matrix(nm, nrow = nrow(mat), ncol = ncol(mat))
  }
  return(nm)
}

#' @keywords internal
#' @noRd
mean_matrix <- function(mat) {
  if (nrow(mat) == 0 || ncol(mat) == 0) {
    return(rep(0, ncol(mat)))
  }
  apply(mat, 2, function(col) mean(col, na.rm = TRUE))
}

#' @keywords internal
#' @noRd
sd_matrix <- function(mat) {
  if (nrow(mat) == 0 || ncol(mat) == 0) {
    return(rep(1, ncol(mat)))
  }
  apply(mat, 2, function(col) {
    s <- sd(col, na.rm = TRUE)
    if (is.na(s) || s == 0) 1 else s
  })
}

#' @keywords internal
#' @noRd
normalize_matrix_known_mean_variance <- function(mat, mean, sd) {
  if (nrow(mat) == 0 || ncol(mat) == 0) {
    return(matrix(numeric(0), nrow = nrow(mat), ncol = ncol(mat)))
  }
  nm <- sapply(1:NCOL(mat), function(col) {
    (mat[, col] - mean[col]) / sd[col]
  })
  if (is.null(dim(nm))) {
    nm <- matrix(nm, nrow = nrow(mat), ncol = ncol(mat))
  }
  return(nm)
}

#' @keywords internal
#' @noRd
normalize_nested_list <- function(nested_list) {
  lapply(nested_list, function(subgroup) lapply(subgroup, normalize_matrix))
}

#' @keywords internal
#' @noRd
mean_nested_list <- function(nested_list) {
  lapply(nested_list, function(subgroup) lapply(subgroup, mean_matrix))
}

#' @keywords internal
#' @noRd
sd_nested_list <- function(nested_list) {
  lapply(nested_list, function(subgroup) lapply(subgroup, sd_matrix))
}

#' @keywords internal
#' @noRd
normalize_nested_list_known_mean_sd <- function(nested_list, mean_nested_list, sd_nested_list) {
  mapply(function(x, y, z) {
    mapply(normalize_matrix_known_mean_variance, x, y, z, SIMPLIFY = FALSE)
  }, nested_list, mean_nested_list, sd_nested_list, SIMPLIFY = FALSE)
}

test_that("normalize_matrix standardizes columns to mean 0, sd 1", {
  m <- matrix(rnorm(50, mean = 5, sd = 3), ncol = 5)
  z <- IntegMultiReg:::normalize_matrix(m)
  expect_equal(unname(colMeans(z)), rep(0, 5), tolerance = 1e-8)
  expect_equal(unname(apply(z, 2, stats::sd)), rep(1, 5), tolerance = 1e-8)
})

test_that("normalize_matrix handles constant and empty columns gracefully", {
  m <- cbind(rnorm(10), rep(2, 10))
  z <- IntegMultiReg:::normalize_matrix(m)
  expect_true(all(z[, 2] == 0))          # zero-variance column -> all zeros
  empty <- IntegMultiReg:::normalize_matrix(matrix(numeric(0), nrow = 0, ncol = 3))
  expect_equal(dim(empty), c(0L, 3L))
})

test_that("known-mean/sd normalization inverts the training transform", {
  m <- matrix(rnorm(40), ncol = 4)
  mu <- IntegMultiReg:::mean_matrix(m)
  s <- IntegMultiReg:::sd_matrix(m)
  z <- IntegMultiReg:::normalize_matrix_known_mean_variance(m, mu, s)
  expect_equal(z, IntegMultiReg:::normalize_matrix(m), tolerance = 1e-8)
})

test_that("subgroup_data partitions subjects by availability pattern", {
  d <- IntegMultiReg:::subgroup_data(simIMR$outcome.binary, simIMR$covariates,
                                     simIMR$platforms)
  expect_named(d, c("outcome", "covariate", "platform_data"))
  expect_setequal(names(d$platform_data), c("011", "100", "101", "111"))
})

test_that("subgroup_data aligns outcome, covariate and platform rows by id", {
  set.seed(99)
  platforms <- lapply(simIMR$platforms, function(x) x[sample(nrow(x)), ])
  outcome <- simIMR$outcome.binary[sample(nrow(simIMR$outcome.binary)), ]
  cov <- simIMR$covariates[sample(nrow(simIMR$covariates)), ]

  d <- IntegMultiReg:::subgroup_data(outcome, cov, platforms)

  for (pat in names(d$platform_data)) {
    ids <- d$outcome[[pat]]$id
    expect_identical(d$covariate[[pat]]$id, ids)
    for (platform in d$platform_data[[pat]]) {
      if (nrow(platform) > 0L) {
        expect_identical(platform$id, ids)
      }
    }
  }
})

## Shared base-graphics helpers used by the IMR plotting functions.

#' @keywords internal
#' @noRd
.imr_plot_take_alias <- function(dots, alias, value) {
  if (!is.null(dots[[alias]])) {
    value <- dots[[alias]]
    dots[[alias]] <- NULL
  }
  list(value = value, dots = dots)
}

#' @keywords internal
#' @noRd
.imr_plot_cex <- function(dots, base_cex = 1, cex_axis = NULL,
                          cex_lab = NULL, cex_main = NULL,
                          cex_names = NULL, cex_legend = NULL,
                          cex_values = NULL,
                          defaults = list(axis = 1, lab = 1.15,
                                          main = 1.2, names = 1,
                                          legend = 0.95, values = 0.95)) {
  base_cex <- .imr_check_numeric_vector(
    base_cex, "base_cex", length = 1, positive = TRUE
  )

  r <- .imr_plot_take_alias(dots, "cex.axis", cex_axis)
  cex_axis <- r$value; dots <- r$dots
  r <- .imr_plot_take_alias(dots, "cex.lab", cex_lab)
  cex_lab <- r$value; dots <- r$dots
  r <- .imr_plot_take_alias(dots, "cex.main", cex_main)
  cex_main <- r$value; dots <- r$dots
  r <- .imr_plot_take_alias(dots, "cex.names", cex_names)
  cex_names <- r$value; dots <- r$dots
  r <- .imr_plot_take_alias(dots, "cex.legend", cex_legend)
  cex_legend <- r$value; dots <- r$dots
  r <- .imr_plot_take_alias(dots, "cex.values", cex_values)
  cex_values <- r$value; dots <- r$dots

  out <- list(
    axis = if (is.null(cex_axis)) defaults$axis * base_cex else cex_axis,
    lab = if (is.null(cex_lab)) defaults$lab * base_cex else cex_lab,
    main = if (is.null(cex_main)) defaults$main * base_cex else cex_main,
    names = if (is.null(cex_names)) defaults$names * base_cex else cex_names,
    legend = if (is.null(cex_legend)) {
      defaults$legend * base_cex
    } else {
      cex_legend
    },
    values = if (is.null(cex_values)) {
      defaults$values * base_cex
    } else {
      cex_values
    },
    dots = dots
  )

  for (nm in setdiff(names(out), "dots")) {
    out[[nm]] <- .imr_check_numeric_vector(
      out[[nm]], paste0("cex_", nm), length = 1, positive = TRUE
    )
  }
  out
}

#' @keywords internal
#' @noRd
.imr_plot_par <- function(mar, mgp, default_mar, default_mgp = c(2.7, 0.8, 0)) {
  if (is.null(mar)) mar <- default_mar
  if (is.null(mgp)) mgp <- default_mgp
  mar <- .imr_check_numeric_vector(
    mar, "mar", length = 4, nonnegative = TRUE
  )
  mgp <- .imr_check_numeric_vector(
    mgp, "mgp", length = 3, nonnegative = TRUE
  )
  list(mar = mar, mgp = mgp)
}

#' @keywords internal
#' @noRd
.imr_plot_palette <- function(n, col = NULL, palette = "Dark 3") {
  if (n < 1L) return(character(0))
  if (!is.null(col)) {
    if (!is.atomic(col) || length(col) == 0L) {
      .imr_abort("`col` must contain at least one colour.")
    }
    return(rep_len(col, n))
  }
  if (!is.character(palette) || length(palette) != 1L || is.na(palette)) {
    .imr_abort("`palette` must be a single character string.")
  }
  switch(tolower(palette),
    platform = rep_len(.imr_plot_platform_colours(), n),
    platforms = rep_len(.imr_plot_platform_colours(), n),
    subgroup = grDevices::colorRampPalette(.imr_plot_subgroup_colours())(n),
    subgroups = grDevices::colorRampPalette(.imr_plot_subgroup_colours())(n),
    heatmap = grDevices::colorRampPalette(.imr_plot_heatmap_colours())(n),
    grey = .imr_plot_greys(n),
    gray = .imr_plot_greys(n),
    grDevices::hcl.colors(n, palette = palette)
  )
}

#' @keywords internal
#' @noRd
.imr_plot_greys <- function(n = 64) {
  grDevices::colorRampPalette(c("#FAFAFA", "#1F1F1F"))(n)
}

#' @keywords internal
#' @noRd
.imr_plot_platform_colours <- function() {
  ## Muted academic palette: distinct, print-friendly, and colourblind aware.
  c("#2F5D7C", "#8A5A44", "#557A60", "#6D5A7D",
    "#8A7D4E", "#5B7E91", "#6F6F6F", "#3F4A52")
}

#' @keywords internal
#' @noRd
.imr_plot_subgroup_colours <- function() {
  c("#D9E2E7", "#A7B9C6", "#5F7F99")
}

#' @keywords internal
#' @noRd
.imr_plot_heatmap_colours <- function() {
  c("#F9FAFA", "#E2E9ED", "#A8B9C6", "#4E6475")
}

#' @keywords internal
#' @noRd
.imr_plot_reference_colour <- function() {
  "#707070"
}

#' @keywords internal
#' @noRd
.imr_plot_trace_colour <- function() {
  "#2B2B2B"
}

test_that("kircIMR matches the reduced TCGA-KIRC example structure", {
  data("kircIMR", package = "IntegMultiReg")

  expect_named(kircIMR$platforms, c("mrna", "mirna", "methylation"))
  expect_equal(sapply(kircIMR$platforms, ncol), c(mrna = 51L, mirna = 31L,
                                                  methylation = 51L))
  expect_equal(ncol(kircIMR$covariates), 5L)
  expect_equal(names(kircIMR$outcome.survival), c("id", "time", "status"))
  expect_equal(kircIMR$outcome, kircIMR$outcome.survival)

  all_ids <- unname(unlist(c(
    list(kircIMR$covariates$id, kircIMR$outcome.survival$id),
    lapply(kircIMR$platforms, `[[`, "id")
  )))
  expect_true(all(grepl("^KIRC[0-9]{3}$", all_ids)))
  expect_false(any(grepl("^TCGA-", all_ids)))
  expect_match(kircIMR$source$data_access, "public UCSC Xena")
  expect_match(kircIMR$source$data_access, "controlled-access")
  expect_match(kircIMR$source$id_policy, "package-internal")
  expect_match(kircIMR$source$id_policy, "not attempt participant re-identification")

  expect_equal(as.integer(kircIMR$model_subgroup_sizes),
               c(63L, 139L, 147L, 172L))
  expect_equal(names(kircIMR$model_subgroup_sizes),
               c("011", "101", "001", "111"))
  expect_equal(kircIMR$paper_alignment$reference_screened_features,
               c(mrna = 776L, mirna = 91L, methylation = 729L))
})

---
title: "Integrative Bayesian Multi-Platform Regression with IntegMultiReg"
author: "Sinian Zhang, Thierry Chekouo"
output:
  rmarkdown::html_vignette:
    toc: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{Integrative Bayesian Multi-Platform Regression with IntegMultiReg}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.2
)
set.seed(1)
```

# Overview

`IntegMultiReg` implements the *integrative multi-regression* (IMR) model of
Chekouo, Stingo, Doecke and Do (2017) and extends it from time-to-event
outcomes to continuous (Gaussian) and binary (probit) outcomes.  The central
idea is to integrate **all** available subjects across several molecular
platforms, even when many subjects are measured on only a subset of the
platforms.

Given $K$ platforms, subjects are partitioned into the $2^K-1$ non-empty
**availability subgroups** of a $K$-set Venn diagram.  One regression model is
built per subgroup, and information is shared across availability subgroups
through

* **non-local (product moment) priors** on the regression coefficients, which
  sharply penalise small effects and yield parsimonious models, and
* a **Markov random field (MRF) prior** on the variable-selection indicators,
  which encourages the same biomarkers to be selected across availability
  subgroups.

```{r load}
library(IntegMultiReg)
data("simIMR", package = "IntegMultiReg")
sapply(simIMR$platforms, dim)
```

The bundled `simIMR` data set has three platforms (`genomic`, `proteomic`,
`metabolomic`) measured on overlapping but partially missing sets of subjects,
clinical covariates for everyone, and three outcome types generated from the
same latent signal.  The truly associated features are recorded in
`simIMR$truth`.

# Real TCGA-KIRC example

The package also includes `kircIMR`, a reduced real-data example derived from
public UCSC Xena TCGA kidney renal clear cell carcinoma (KIRC) sampleMap files,
not from controlled-access TCGA/GDC files.  It mirrors the case-study structure
of Chekouo et al. (2017): mRNA expression, miRNA expression and DNA methylation
measured on overlapping patient sets, clinical covariates, and a right-censored
survival outcome.  Patient IDs are package-internal labels such as `KIRC001`;
the package does not distribute TCGA barcodes, and users should not attempt
participant re-identification or linkage to external resources.

```{r load-kirc}
data("kircIMR", package = "IntegMultiReg")
sapply(kircIMR$platforms, dim)
kircIMR$model_subgroup_sizes
```

The original Biometrics analysis used a much larger screened panel
(776 mRNA, 91 miRNA and 729 methylation features) and long MCMC chains.  The
package object keeps a smaller Cox-screened panel (50, 30 and 50 features) so
that examples remain lightweight while retaining the same scientific data
structure.  A short survival fit follows the same call pattern:

```{r kirc-fit, eval = FALSE}
kirc_fit <- imr(
  platform_data_list = kircIMR$platforms,
  outcome            = kircIMR$outcome.survival,
  cov                = kircIMR$covariates,
  type_outcome       = "right.censored",
  nu                 = c(-4, -3, -4),
  sample_mcmc        = c(4000, 1000),
  ssize              = 30,
  seed               = 1
)
summary(kirc_fit)
plot_top_features(kirc_fit, top = 12)
```

# Fitting the model

The workhorse is `imr()`.  Here we fit a binary (probit) outcome with a short
MCMC run for illustration; in practice use longer chains.

```{r fit}
fit <- imr(
  platform_data_list = simIMR$platforms,
  outcome            = simIMR$outcome.binary,
  cov                = simIMR$covariates,
  type_outcome       = "binary",
  nu                 = c(-4, -3, -4),
  sample_mcmc        = c(1500, 500),
  ssize              = 30,
  seed               = 1
)
fit
```

The four modelled availability subgroups correspond to the availability patterns
`011` (genomic + proteomic), `111` (all three), `101` (genomic + metabolomic)
and `100` (metabolomic only); the bitstring digits run from the first platform
(right) to the last (left).  `plot_subgroup_sizes()` shows how the sample splits
across them.

```{r plot-sizes, fig.height = 3.6}
plot_subgroup_sizes(fit)
```

# Inspecting selected biomarkers

`summary()` ranks, per platform, the features whose marginal posterior
inclusion probability (mPIP) exceeds a threshold.

```{r summary}
summary(fit, threshold = 0.5)
```

The posterior inclusion probabilities themselves are available through `coef()`
(a list of subgroup $\times$ feature matrices) and can be visualised either as
per-platform heatmaps or as a single ranked bar chart of the most strongly
supported features across all platforms.

```{r plot-selection}
plot(fit, type = "selection", platform = 1)
```

```{r plot-top, fig.height = 4}
plot_top_features(fit, top = 8)
```

The estimated MRF interaction parameters, which measure how strongly each pair
of subgroups shares biomarkers, are shown with `type = "theta"`, and the
log-posterior trace with `type = "trace"`.

```{r plot-trace}
plot(fit, type = "trace")
```

# Predicting new subjects

`predict()` routes new subjects to the appropriate availability subgroup,
standardises their features with the training scaling, and performs Bayesian
model averaging.  When the new platforms are supplied in the same order as at
training, `platform_names` can be omitted.  Binary outcomes are returned on the
probability scale.

```{r predict}
new_x <- simIMR$platforms$genomic[1:20, ]
new_p <- simIMR$platforms$proteomic[1:20, ]
pred <- predict(fit, newdata = list(new_x, new_p),
                covariates = simIMR$covariates)
head(pred[["model:011"]])
```

# Assessing predictive performance

`cv_imr()` runs repeated $K$-fold splits using the fitted MCMC samples and
reports the accuracy measure appropriate to the outcome type (AUC for binary,
the concordance index for survival, mean squared error for continuous), overall
and within each availability subgroup.  The MCMC sampler is not re-run inside
each fold; to compare IMR with BMS, fit a separate `method = "BMS"` object.

```{r cv}
cv <- cv_imr(fit, k = 5, rounds = 3)
attr(cv, "metric")
round(colMeans(cv$total_cindex), 3)
```

# Other outcome types

The same interface handles continuous and right-censored outcomes; only
`type_outcome` and the `outcome` data frame change.

```{r other-types, eval = FALSE}
# continuous (Gaussian) outcome
fit_c <- imr(
  simIMR$platforms, simIMR$outcome.continuous, cov = simIMR$covariates,
  type_outcome = "continuous", nu = c(-4, -3, -4),
  sample_mcmc = c(1500, 500), ssize = 30, seed = 1)

# right-censored survival outcome
fit_s <- imr(
  simIMR$platforms, simIMR$outcome.survival, cov = simIMR$covariates,
  type_outcome = "right.censored", nu = c(-4, -3, -4),
  sample_mcmc = c(1500, 500), ssize = 30, seed = 1)
```

# References

Chekouo T, Stingo FC, Doecke JD, Do K-A (2017). "A Bayesian Integrative
Approach for Multi-Platform Genomic Data: A Kidney Cancer Case Study."
*Biometrics*, **73**(2), 615--624. <doi:10.1111/biom.12587>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_subgroup_sizes}
\alias{plot_subgroup_sizes}
\title{Plot the Availability Subgroup Sizes of an IMR Fit}
\usage{
plot_subgroup_sizes(
  object,
  base_cex = 1,
  cex_axis = NULL,
  cex_lab = NULL,
  cex_main = NULL,
  cex_values = NULL,
  col = NULL,
  palette = "subgroup",
  show_values = TRUE,
  ylim = NULL,
  mar = NULL,
  mgp = NULL,
  ...
)
}
\arguments{
\item{object}{A fitted object of class `"imr"` returned by [imr()].}

\item{base_cex}{Overall text-size multiplier. The `cex_*` arguments default
to values derived from this multiplier (default `1`).}

\item{cex_axis}{Axis-label size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{cex_lab}{Axis-title size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{cex_main}{Main-title size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{cex_values}{Bar-value label size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{col}{Optional bar colours. If `NULL`, colours are generated from
`palette`.}

\item{palette}{Palette name used when `col = NULL`; `"subgroup"` gives the
package's standard neutral grey-blue subgroup colours (default).}

\item{show_values}{Logical; should sample sizes be printed above the bars
(default `TRUE`)?}

\item{ylim}{Optional numeric vector of length two giving the vertical axis
limits. If `NULL`, limits are chosen from the subgroup sizes.}

\item{mar, mgp}{Optional graphical margin and axis-title placement vectors
passed to [graphics::par()] for finer layout control.}

\item{...}{Further graphical parameters passed to [graphics::barplot()].}
}
\value{
Invisibly, the named integer vector of subgroup sizes.
}
\description{
Draws a bar chart of the number of subjects in each modelled
availability subgroup (the non-empty regions of the Venn diagram), giving a
quick picture of how the sample is distributed across subgroups.
}
\examples{
\donttest{
data("simIMR", package = "IntegMultiReg")
fit <- imr(
  platform_data_list = simIMR$platforms, outcome = simIMR$outcome,
  cov = simIMR$covariates, type_outcome = "binary",
  nu = c(-4, -3, -4), sample_mcmc = c(200, 100), ssize = 5, seed = 1
)
plot_subgroup_sizes(fit)
}
}
\seealso{
[imr()], [plot.imr()], [plot_top_features()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.imr}
\alias{print.imr}
\title{Print Method for IMR Fits}
\usage{
\method{print}{imr}(x, threshold = 0.5, rank = FALSE, top = 5, ...)
}
\arguments{
\item{x}{A fitted object of class `"imr"`.}

\item{threshold}{Inclusion-probability threshold used to count selected
features (default `0.5`).}

\item{rank}{Logical; if `TRUE`, print a short ranked feature table for each
platform (default `FALSE`).}

\item{top}{Integer; when `rank = TRUE`, the number of top-ranked features to
show per platform (default `5`).}

\item{...}{Unused; present for S3 compatibility.}
}
\value{
`x`, invisibly.
}
\description{
Prints a compact overview of the fit, including a platform key (`P1`,
`P2`, ...) that decodes the availability-subgroup bitstrings.  Optionally,
it can also print the top-ranked features per platform, ranked by each
feature's maximum mPIP over availability subgroups containing that platform.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R, R/wrappers.R
\name{summary.imr}
\alias{summary.imr}
\alias{print.summary.imr}
\alias{summary_imr}
\title{Summarize an IMR Fit}
\usage{
\method{summary}{imr}(object, threshold = 0.5, ...)

\method{print}{summary.imr}(x, ...)

summary_imr(object, ...)
}
\arguments{
\item{object}{A fitted object of class `"imr"`.}

\item{threshold}{Inclusion-probability threshold for selection (default
`0.5`).}

\item{...}{Unused; present for S3 compatibility.}

\item{x}{A `"summary.imr"` object.}
}
\value{
An object of class `"summary.imr"`: a list with the run metadata and,
  for each platform, a data frame of selected features with their maximum
  mPIP and the subgroup achieving it.
}
\description{
Produces a per-platform summary of the selected features (those whose
marginal posterior inclusion probability exceeds `threshold` in at least one
subgroup), ranked by their maximum inclusion probability.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv.R
\name{cv_imr}
\alias{cv_imr}
\title{Cross-Validated Predictive Accuracy of an IMR Fit}
\usage{
cv_imr(
  object,
  k = 5,
  rounds = 2,
  method = NULL,
  max_models = 100,
  verbose = FALSE
)
}
\arguments{
\item{object}{A fitted object of class `"imr"` returned by [imr()].}

\item{k}{Integer number of cross-validation folds per round (default `5`).
Must be at least `2`, and each availability subgroup must contain at least
`k` subjects.}

\item{rounds}{Integer number of independent cross-validation rounds to
average over (default `2`).  Must be positive.}

\item{method}{Optional compatibility argument.  If supplied, it must match
the method stored in `object`; `cv_imr()` cannot turn an IMR fit into a BMS
fit or vice versa.}

\item{max_models}{Integer maximum number of selection models used for
Bayesian model averaging (default `100`).  Must be positive.}

\item{verbose}{Logical; if `TRUE`, print the C routine's cross-validation
diagnostics.  Defaults to `FALSE`.}
}
\value{
A list with two numeric matrices, each of dimension
  `rounds` x `(n_subgroups + 1)`, whose last column corresponds to all
  subjects pooled and whose remaining columns are named by the availability
  subgroup bitstrings:
  \item{total_cindex}{Accuracy computed on the pooled cross-validated
    predictions within each availability subgroup (and overall).}
  \item{subset_cindex}{Accuracy computed fold-by-fold within each
    availability subgroup.}
  The accuracy metric (`"C-index"`, `"AUC"` or `"MSE"`) is recorded in the
  `"metric"` attribute of the returned list.
}
\description{
`cv_imr()` evaluates predictive accuracy of a model fitted with [imr()] using
repeated \eqn{K}-fold splits and Bayesian model averaging over the fitted
MCMC samples.  The sampler is not re-run inside each training fold; fit a
separate object with `method = "BMS"` if a no-borrowing comparison is needed.
The accuracy measure depends on the outcome type: the concordance index
(C-index) for right-censored outcomes, the area under the ROC curve (AUC) for
binary outcomes, and the mean squared error (MSE) for continuous outcomes.
}
\examples{
\donttest{
data("simIMR", package = "IntegMultiReg")
fit <- imr(
  platform_data_list = simIMR$platforms, outcome = simIMR$outcome,
  cov = simIMR$covariates, type_outcome = "binary",
  nu = c(-4, -3, -4), sample_mcmc = c(200, 100), ssize = 5, seed = 1
)
cv <- cv_imr(fit, k = 5, rounds = 2)
cv$total_cindex
}
}
\seealso{
[imr()], [predict.imr()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IntegMultiReg-package.R
\docType{package}
\name{IntegMultiReg-package}
\alias{IntegMultiReg}
\alias{IntegMultiReg-package}
\title{IntegMultiReg: Integrative Bayesian Multiple Regression for Multi-Platform Biomarkers}
\description{
The \pkg{IntegMultiReg} package implements the integrative multi-regression
(IMR) approach of Chekouo et al. (2017) and extends it from time-to-event
outcomes to continuous (Gaussian) and binary (probit) outcomes.  Subjects
are partitioned into availability subgroups according to which platforms
they have measured, one regression model is built per subgroup, and
information is shared across availability subgroups through a Markov random
field prior on the
variable-selection indicators combined with non-local priors on the
regression coefficients.
}
\details{
The main entry points are:
\describe{
  \item{\code{\link{imr}}}{Fit the model by MCMC and return an object of
    class \code{"imr"}.}
  \item{\code{\link{predict.imr}}}{Predict outcomes for new subjects.}
  \item{\code{\link{cv_imr}}}{Assess predictive performance by
    cross-validation.}
}
Standard methods \code{\link{print.imr}}, \code{\link{summary.imr}},
\code{\link{coef.imr}}, \code{\link{plot.imr}} and \code{\link{predict.imr}}
are provided for the fitted object, together with the stand-alone displays
\code{\link{plot_top_features}} and \code{\link{plot_subgroup_sizes}}.

Jianfeng Wang initiated the R interface that calls the C implementation.
}
\references{
Chekouo T, Stingo FC, Doecke JD, Do K-A (2017). "A Bayesian Integrative
Approach for Multi-Platform Genomic Data: A Kidney Cancer Case Study."
\emph{Biometrics}, \strong{73}(2), 615--624. \doi{10.1111/biom.12587}
}
\author{
\strong{Maintainer}: Thierry Chekouo \email{tchekouo@umn.edu} (\href{https://orcid.org/0000-0002-3594-9182}{ORCID})

Authors:
\itemize{
  \item Sinian Zhang \email{zhan9381@umn.edu}
  \item Jianfeng Wang \email{wan01188@umn.edu}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R, R/wrappers.R
\name{plot.imr}
\alias{plot.imr}
\alias{plot_imr}
\title{Plot Method for IMR Fits}
\usage{
\method{plot}{imr}(
  x,
  type = c("selection", "theta", "trace"),
  platform = NULL,
  base_cex = 1,
  cex_axis = NULL,
  cex_lab = NULL,
  cex_main = NULL,
  col = NULL,
  palette = NULL,
  legend = TRUE,
  legend_width = 0.28,
  mar = NULL,
  mgp = NULL,
  ...
)

plot_imr(x, ...)
}
\arguments{
\item{x}{A fitted object of class `"imr"`.}

\item{type}{Character; one of `"selection"` (default), `"theta"` or
`"trace"`.}

\item{platform}{Optional integer vector selecting which platforms to display
for the `"selection"` and `"theta"` plots; defaults to all platforms.}

\item{base_cex}{Overall text-size multiplier. The `cex_*` arguments default
to values derived from this multiplier (default `1`).}

\item{cex_axis}{Axis-label size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{cex_lab}{Axis-title size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{cex_main}{Main-title size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{col}{Optional colours. For heatmaps this is the colour scale; for
trace plots this is the line colour.}

\item{palette}{Optional heatmap palette name used when `col = NULL`.
Selection plots default to `"grey"` to preserve the mPIP intensity scale;
theta plots default to the muted grey-blue `"heatmap"` palette.}

\item{legend}{Logical; for `"selection"` plots, should the mPIP intensity
legend be drawn (default `TRUE`)?}

\item{legend_width}{Relative width of the intensity-legend panel for
`"selection"` plots (default `0.28`).}

\item{mar, mgp}{Optional graphical margin and axis-title placement vectors
passed to [graphics::par()] for finer layout control.}

\item{...}{Further graphical parameters passed to the underlying plotting
functions.}
}
\value{
`NULL`, invisibly; called for the side effect of producing a plot.
}
\description{
Visualizes a fitted `"imr"` object.  Three plot types are available:
\describe{
  \item{`"selection"`}{Heatmap of the marginal posterior inclusion
    probabilities (mPIP), one panel per platform, with features on the
    horizontal axis, availability subgroups on the vertical axis and a small
    intensity legend showing that darker values are closer to 1.}
  \item{`"theta"`}{Heatmap of the posterior mean MRF interaction parameters
    between availability subgroups, one panel per platform.}
  \item{`"trace"`}{Trace plot of the log-posterior across MCMC iterations.}
}
See [plot_top_features()] and [plot_subgroup_sizes()] for two further ready
made displays.
}
\seealso{
[imr()], [plot_top_features()], [plot_subgroup_sizes()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R, R/wrappers.R
\name{predict.imr}
\alias{predict.imr}
\alias{predict_imr}
\title{Predict Outcomes for New Subjects}
\usage{
\method{predict}{imr}(
  object,
  newdata,
  platform_names = NULL,
  covariates = NULL,
  max_models = 100,
  verbose = FALSE,
  ...
)

predict_imr(object, newdata, ...)
}
\arguments{
\item{object}{A fitted object of class `"imr"` returned by [imr()].}

\item{newdata}{A list of data frames with the new platform measurements.
Each data frame must include `id` as the first column, followed by finite
numeric feature columns matching the corresponding training platform.}

\item{platform_names}{A character vector giving, for each element of
`newdata`, the index (`"1"`, `"2"`, ...) of the corresponding training
platform.  Defaults to `NULL`, meaning the elements of `newdata` are taken
to be in the same order as the platforms supplied to [imr()].}

\item{covariates}{An optional data frame of clinical covariates for the test
subjects, including `id` as the first column.  Required when the model was
fitted with covariates and ignored with a warning when it was not.}

\item{max_models}{Integer; the maximum number of distinct selection models
(gamma configurations) used for Bayesian model averaging.  Default `100`.}

\item{verbose}{Logical; if `TRUE`, print the C routine's diagnostics.
Defaults to `FALSE`.}

\item{...}{Unused; present for S3 compatibility.}
}
\value{
A named list with one data frame per active availability subgroup
  model. Each data frame has columns `id` (subject identifier) and `predict`
  (predicted value or probability, rounded to three decimals).
}
\description{
`predict()` method for objects of class `"imr"` produced by [imr()].  New
subjects are matched to the availability subgroup models learned during
training, missing platforms are handled automatically, and the
test features are standardized with the training set's centring and scaling
factors before Bayesian model averaging.
}
\details{
Predictions are only produced for subjects observed on at least one platform
(and, when `covariates` is supplied, with covariate data).  For `"binary"`
outcomes the returned `predict` column is a probability obtained through the
probit link (`pnorm`); for `"continuous"` and `"right.censored"` outcomes it
is the predicted (latent) response.
}
\examples{
\donttest{
data("simIMR", package = "IntegMultiReg")
fit <- imr(
  platform_data_list = simIMR$platforms, outcome = simIMR$outcome,
  cov = simIMR$covariates, type_outcome = "binary",
  nu = c(-4, -3, -4), sample_mcmc = c(200, 100), ssize = 5, seed = 1
)
new_x <- simIMR$platforms[[1]][1:10, ]
new_z <- simIMR$platforms[[2]][1:7, ]
predict(fit, newdata = list(new_x, new_z), covariates = simIMR$covariates)
}
}
\seealso{
[imr()], [cv_imr()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{simIMR}
\alias{simIMR}
\title{Simulated Multi-Platform Example Data}
\format{
A named list with the components:
\describe{
  \item{platforms}{A list of three data frames, `genomic` (20 features,
    subjects 1--240), `proteomic` (10 features, subjects 1--180) and
    `metabolomic` (8 features, subjects 121--300).  Each has an `id` column
    followed by the feature columns.}
  \item{covariates}{A data frame of three clinical covariates (`age`, `sex`,
    `stage`) with an `id` column, available for all 300 subjects.}
  \item{outcome}{The binary outcome data frame (alias of
    `outcome.binary`), used as the canonical demonstration outcome.}
  \item{outcome.binary}{Data frame with `id` and a 0/1 response `y`.}
  \item{outcome.continuous}{Data frame with `id` and a numeric response `y`.}
  \item{outcome.survival}{Data frame with `id`, observed `time` and event
    indicator `status` (1 = event, 0 = censored).}
  \item{truth}{A list giving, per platform, the column indices of the
    features with non-zero effects used to generate the outcomes.}
}
}
\usage{
simIMR
}
\description{
A small simulated data set illustrating the integrative multi-regression
(IMR) setting: three molecular platforms measured on overlapping but
partially missing sets of subjects, clinical covariates available for
everyone, and three outcome types (continuous, binary and right-censored)
driven by the same latent signal.
}
\details{
The platforms are observed on the following subjects, which induces four
non-empty availability subgroups (bitstrings over genomic/proteomic/
metabolomic): `011` (120 subjects), `111` (60), `101` (60) and `100` (60).

The generating script is provided in `data-raw/make_simIMR.R`.
}
\examples{
data("simIMR", package = "IntegMultiReg")
sapply(simIMR$platforms, dim)
str(simIMR$truth)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{imr}
\alias{imr}
\title{Fit the Integrative Bayesian Multi-Platform Regression Model}
\usage{
imr(
  platform_data_list,
  outcome,
  cov = NULL,
  type_outcome = c("right.censored", "binary", "continuous"),
  method = c("IMR", "BMS"),
  ssize = 30,
  nu = rep(-3, length(platform_data_list)),
  hh = 0.087,
  h0 = 10000,
  sig_alpha_psi = c(0.001, 0.001),
  thet_alph_bet = c(40, 10),
  sample_mcmc = c(2000, 1000),
  seed = NULL,
  verbose = FALSE
)
}
\arguments{
\item{platform_data_list}{A list of data frames, one per platform.  Each
data frame must contain an `id` column (the subject identifier, taken to be
the first column); the remaining columns are finite numeric features
measured on that platform.  Subject identifiers must be unique within each
data frame.  The `id` column links subjects across platforms and to the
outcome and covariate data.}

\item{outcome}{A data frame containing an `id` column and the response.  For
`type_outcome = "right.censored"` it must also contain the event/censoring
time and a censoring indicator (three columns in total).  For `"binary"`
the response must be coded 0/1 and for `"continuous"` it is a numeric
response (two columns in total).  The `id` column must be first and unique.}

\item{cov}{An optional data frame of clinical covariates including an `id`
column followed by finite numeric covariates.  Covariates are always
included in every regression (they are not subject to selection).  Defaults
to `NULL` (no covariates).}

\item{type_outcome}{Character string specifying the outcome type, one of
`"right.censored"` (default), `"binary"` or `"continuous"`.}

\item{method}{Character string specifying the method, `"IMR"` (default) for
the integrative model that shares information across subgroups via the MRF
prior, or `"BMS"` for the non-integrative Bayesian multi-step variant that
fits each subgroup independently (MRF interaction parameters set to zero).}

\item{ssize}{Minimum availability subgroup size for a subgroup to be
modelled. Subgroups with at most `ssize` subjects are dropped. Default is
`30`.}

\item{nu}{A numeric vector of prior log-odds of inclusion, one value per
platform, controlling the prior sparsity of the selected features.
Defaults to `rep(-3, length(platform_data_list))`.}

\item{hh}{Scale of the non-local (product moment) prior on the slab
regression effects (default `0.087`).}

\item{h0}{Prior variance of the intercept (default `10000`).}

\item{sig_alpha_psi}{Length-2 numeric vector with the shape and rate of the
inverse-gamma prior on the response error variance (default
`c(0.001, 0.001)`).  Ignored when `type_outcome = "binary"`: a probit model
has residual variance fixed at 1 for identifiability, so the error variance
is pinned to 1 rather than estimated.}

\item{thet_alph_bet}{Length-2 numeric vector with the shape and rate of the
gamma prior on the MRF interaction parameters theta, which borrow
information across subgroups (default `c(40, 10)`).}

\item{sample_mcmc}{An integer vector `c(n_retained, n_burnin)` giving the
number of post-burn-in draws to retain and the number of burn-in
iterations to discard.  The sampler runs `n_retained + n_burnin`
iterations in total, so the returned `log_posterior` has
`sum(sample_mcmc)` entries.  Default is `c(2000, 1000)`.}

\item{seed}{Optional integer seed for the sampler.  If `NULL` (the default) a
seed is drawn from the current R RNG state, so a run is reproducible
whenever [set.seed()] is called beforehand or an explicit `seed` is passed.}

\item{verbose}{Logical; if `TRUE`, print the sampler's progress and
diagnostics to the console.  Defaults to `FALSE` (quiet).}
}
\value{
An object of class `"imr"`: a list with components
\item{gam_mean}{List (one matrix per platform) of posterior mean
  variable-selection probabilities; rows index the subgroups containing the
  platform and columns index the platform features.}
\item{theta_mean}{List (one matrix per platform) of posterior mean MRF
  interaction parameters between subgroups.}
\item{estimate_latent_y}{List (one vector per subgroup) of posterior mean
  latent responses, useful for censored or binary data.}
\item{log_posterior}{Numeric vector of log-posterior values across all MCMC
  iterations (burn-in included).}
\item{gam_sample}{Post-burn-in MCMC samples of the selection indicators.}
\item{theta_sample}{Post-burn-in MCMC samples of the MRF interaction
  parameters (absent when `method = "BMS"`).}
\item{list_hyperpara, data1, data2}{Hyper-parameters and pre-processed data
  retained for prediction and cross-validation.}
\item{call, type_outcome, n_platform, platform_names, feature_names,
  covariate_names, model_bitstrings, sample_size, model_platforms,
  platform_models, sample_mcmc, nu, method}{Run metadata used by the print,
  summary, plot and predict methods.}
}
\description{
`imr()` fits the integrative multi-regression (IMR) model of Chekouo et al.
(2017) by Markov chain Monte Carlo (MCMC).
It identifies biomarkers associated with a time-to-event, binary or
continuous outcome while borrowing information across all subjects,
regardless of which platforms each subject has measured.  Subjects are
partitioned into availability subgroups defined by their pattern of platform
availability; one regression is built per subgroup and information is shared
across subgroups through a Markov random field (MRF) prior on the
variable-selection indicators together with non-local priors on the
regression coefficients.  The sampler is implemented in C for efficiency.

The arguments are grouped by the orthogonal aspect of the analysis that each
one controls: the *data* (`platform_data_list`, `outcome`, `cov`), the
*likelihood* (`type_outcome`), the *model* (`method`), subgroup *filtering*
(`ssize`), the *priors* (`nu`, `hh`, `h0`, `sig_alpha_psi`, `thet_alph_bet`),
the *computation* (`sample_mcmc`, `seed`) and the *output* (`verbose`).
}
\details{
All feature and covariate data are standardized internally (mean 0, standard
deviation 1); the centring and scaling factors are stored in the returned
object so that [predict.imr()] can apply the same transformation to new
subjects.  For right-censored outcomes the latent log-survival times of
censored subjects are imputed within the sampler; for binary outcomes a
probit data-augmentation latent variable is sampled.
}
\examples{
\donttest{
data("simIMR", package = "IntegMultiReg")
fit <- imr(
  platform_data_list = simIMR$platforms,
  outcome = simIMR$outcome,
  cov = simIMR$covariates,
  type_outcome = "binary",
  nu = c(-4, -3, -4),
  sample_mcmc = c(200, 100),
  ssize = 5,
  seed = 1
)
fit
}
}
\references{
Chekouo T, Stingo FC, Doecke JD, Do K-A (2017). "A Bayesian Integrative
Approach for Multi-Platform Genomic Data: A Kidney Cancer Case Study."
\emph{Biometrics}, \strong{73}(2), 615--624. \doi{10.1111/biom.12587}
}
\seealso{
[predict.imr()], [cv_imr()], [summary.imr()], [plot.imr()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{kircIMR}
\alias{kircIMR}
\title{Reduced TCGA-KIRC Multi-Platform Survival Example}
\format{
A named list with the components:
\describe{
  \item{platforms}{A list of three data frames, `mrna`, `mirna` and
    `methylation`.  Each has an `id` column containing package-internal
    patient labels followed by screened molecular features.}
  \item{covariates}{Clinical covariates with an `id` column: age at initial
    diagnosis, female sex indicator, pathologic stage and histologic grade.}
  \item{outcome}{Alias of `outcome.survival`, for convenience.}
  \item{outcome.survival}{Data frame with `id`, observed survival/follow-up
    time in days and event indicator `status` (1 = deceased, 0 = censored).}
  \item{feature_screening}{Per-platform feature-ranking information and a
    short description of the screening procedure used to create the reduced
    panel.}
  \item{platform_availability}{Patient-level logical matrix indicating which
    of the three platforms are available.}
  \item{subgroup_sizes}{Counts of the non-empty availability subgroups
    induced by the reduced data.}
  \item{model_subgroup_sizes}{The availability subgroups that would be
    modelled under the default real-data illustration threshold
    `ssize = 30`.}
  \item{paper_alignment}{Metadata describing how the reduced example maps to
    the Biometrics KIRC case study.}
  \item{source}{Source URLs and reference metadata.}
}
}
\usage{
kircIMR
}
\description{
A reduced real-data example derived from public UCSC Xena TCGA kidney renal
clear cell carcinoma (KIRC) data, not from controlled-access TCGA/GDC files.
The data set mirrors the multi-platform structure of Chekouo, Stingo, Doecke
and Do (2017): mRNA expression, miRNA expression and DNA methylation are
observed on overlapping but not identical sets of patients, with clinical
covariates and right-censored survival outcome.
}
\details{
The full TCGA source files are not distributed with the package.  Instead,
this object stores a small screened feature panel suitable for examples and
vignette analyses.  Patient identifiers are package-internal labels
(`KIRC001`, `KIRC002`, ...) rather than TCGA barcodes; no barcode mapping is
distributed with the package.  Users should not attempt participant
re-identification or linkage to external resources.  The generating script is
provided in the source repository under `data-raw/make_kircIMR.R`.

The source data are public UCSC Xena TCGA-KIRC sampleMap files:
mRNA expression (`HiSeqV2`), miRNA expression (`miRNA_HiSeq_gene`), DNA
methylation (`HumanMethylation450`) and the KIRC clinical matrix.  Molecular
features are screened with univariable Cox models after basic filtering, and
missing molecular values in the retained panel are median-imputed within
feature.  The full Biometrics analysis used 776 mRNA, 91 miRNA and 729 DNA
methylation features; this package example retains 50, 30 and 50 features,
respectively, to keep examples lightweight.  The package does not distribute
controlled-access TCGA/GDC data or the full Biometrics/Wiley supplementary
data files.
}
\examples{
data("kircIMR", package = "IntegMultiReg")
sapply(kircIMR$platforms, dim)
kircIMR$subgroup_sizes
kircIMR$model_subgroup_sizes
}
\references{
Chekouo T, Stingo FC, Doecke JD, Do K-A (2017). "A Bayesian
Integrative Approach for Multi-Platform Genomic Data: A Kidney Cancer Case
Study." \emph{Biometrics}, \strong{73}(2), 615--624.
\doi{10.1111/biom.12587}

The Cancer Genome Atlas Research Network; National Cancer Institute Genomic
Data Commons; UCSC Xena.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R, R/wrappers.R
\name{coef.imr}
\alias{coef.imr}
\alias{coef_imr}
\title{Marginal Posterior Inclusion Probabilities of an IMR Fit}
\usage{
\method{coef}{imr}(object, ...)

coef_imr(object, ...)
}
\arguments{
\item{object}{A fitted object of class `"imr"`.}

\item{...}{Unused; present for S3 compatibility.}
}
\value{
A named list with one matrix per platform.  Rows are the subgroups
  containing that platform (labelled by their availability bitstrings) and
  columns are the platform features.
}
\description{
Extracts the posterior mean variable-selection probabilities (the marginal
posterior inclusion probabilities, mPIP) of a fitted model.
}
\seealso{
[imr()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_top_features}
\alias{plot_top_features}
\title{Plot the Top Selected Features of an IMR Fit}
\usage{
plot_top_features(
  object,
  top = 10,
  base_cex = 1,
  cex_names = NULL,
  cex_axis = NULL,
  cex_lab = NULL,
  cex_main = NULL,
  cex_legend = NULL,
  col = NULL,
  palette = "platform",
  reference = 0.5,
  show_source = TRUE,
  legend = TRUE,
  xlim = c(0, 1),
  mar = NULL,
  mgp = NULL,
  ...
)
}
\arguments{
\item{object}{A fitted object of class `"imr"` returned by [imr()].}

\item{top}{Integer; the number of highest-mPIP features to display
(default `10`).}

\item{base_cex}{Overall text-size multiplier. The `cex_*` arguments default
to values derived from this multiplier (default `1`).}

\item{cex_names}{Feature-label size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{cex_axis}{Axis-label size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{cex_lab}{Axis-title size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{cex_main}{Main-title size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{cex_legend}{Legend text size multiplier. If `NULL`, a plot-specific
default derived from `base_cex` is used.}

\item{col}{Optional bar colours. If `NULL`, colours are generated from
`palette`.}

\item{palette}{Palette name used when `col = NULL`; `"platform"` gives the
package's standard muted platform colours (default).}

\item{reference}{Optional vertical reference line; use `NULL` to suppress it
(default `0.5`).}

\item{show_source}{Logical; should each bar be annotated with the
availability subgroup (bitstring) in which the feature attained its maximum
mPIP (default `TRUE`)?}

\item{legend}{Logical; should the platform legend be drawn (default `TRUE`)?}

\item{xlim}{Numeric vector of length two giving the horizontal axis limits
(default `c(0, 1)`).}

\item{mar, mgp}{Optional graphical margin and axis-title placement vectors
passed to [graphics::par()] for finer layout control.}

\item{...}{Further graphical parameters passed to [graphics::barplot()].}
}
\value{
Invisibly, a data frame of the displayed features with columns
  `platform`, `feature`, `mpip` and `subgroup`, where `mpip` is the maximum
  subgroup mPIP for that platform-feature pair and `subgroup` is the
  subgroup where the maximum is attained. Rows are ordered by decreasing
  mPIP.
}
\description{
Draws a horizontal bar chart of the features with the highest marginal
posterior inclusion probability (mPIP).  For each platform-feature pair, the
plotted score is the highest mPIP attained among the availability subgroups
containing that platform; the displayed bars are the largest such scores
across all platforms.  This complements the per-platform heatmap of
`plot(fit, type = "selection")`.
}
\examples{
\donttest{
data("simIMR", package = "IntegMultiReg")
fit <- imr(
  platform_data_list = simIMR$platforms, outcome = simIMR$outcome,
  cov = simIMR$covariates, type_outcome = "binary",
  nu = c(-4, -3, -4), sample_mcmc = c(200, 100), ssize = 5, seed = 1
)
plot_top_features(fit, top = 8)
}
}
\seealso{
[imr()], [plot.imr()], [plot_subgroup_sizes()]
}
